
/**MOD+***********************************************************************/
/* Module:    scaaicb.c                                                      */
/*                                                                           */
/* Purpose:   Sample Call Authorization and Routing Component                */
/*            Authorization Indication processing                            */
/*                                                                           */
/* Copyright (c) 2000, 2001 Data Connection Limited                          */
/* Copyright (c) 2000, 2001 Cisco Systems, Inc. All Rights Reserved.         */
/*                                                                           */
/* All software incorporated in the "OpenOSP Server" (except for those       */
/* portions incorporating third party software specifically identified as    */
/* being licensed under separate license) are licensed under the terms of    */
/* the Cisco Public Source License ver. 1.0 (2000) found in                  */
/* $OSPROOT/license.txt.  Additional copies of the license are available     */
/* from opensource@cisco.com.                                                */
/*                                                                           */
/* Also, before using this software, please see $OSPROOT/x509_ipr.txt for    */
/* information regarding intellectual property rights.                       */
/*                                                                           */
/* $Revision::   1.5                $ $Modtime::   Jan 11 2001 15:56:16   $  */
/*                                                                           */
/**MOD-***********************************************************************/

#include <scaincl.h>

/**PROC+**********************************************************************/
/* Name:      sca_auth_ind_callback                                          */
/*                                                                           */
/* Purpose:   See openosp.h                                                  */
/*                                                                           */
/* Returns:   See openosp.h                                                  */
/*                                                                           */
/* Params:    See openosp.h                                                  */
/*                                                                           */
/*                                                                           */
/* Operation: Verify the authorization, and issue a response with the        */
/*            validity taken from the token                                  */
/*                                                                           */
/**PROC-**********************************************************************/

OSP_RC OSPAPI sca_auth_ind_callback(OSP_CORRELATOR corr,
                                    char *timestamp,
                                    char *role,
                                    OSP_CALL_ID *call_id,
                                    OSP_ADDRESS *source_info,
                                    OSP_ADDRESS *source_alt_list,
                                    OSP_ADDRESS *dest_info,
                                    OSP_ADDRESS *dest_alt_list,
                                    OSP_SERVICE_INFO *service,
                                    OSP_TOKEN *token_list,
                                    OSP_CLIENT_ID *client_id)
{
  /***************************************************************************/
  /* Local Variables                                                         */
  /***************************************************************************/
  OSP_STATUS status;
  OSP_RC rc = OSP_SUCCESS;
  SCA_TOKEN_INFO *token_info;
  char *valid_after;
  char *valid_until;

  TRC_ENTRY("sca_auth_ind_callback");

  /***************************************************************************/
  /* Check parameters                                                        */
  /***************************************************************************/
  TRC_ASSERT((corr != NULL), (TB, "Passed NULL correlator"));
  TRC_ASSERT((source_info != NULL), (TB, "Passed NULL source address"));
  TRC_ASSERT((dest_info != NULL), (TB, "Passed NULL dest address"));
  TRC_DETAIL((TB, "Auth Ind for source '%s' type '%s'",
                                     source_info->address, source_info->type));
  TRC_DETAIL((TB, "Destination address '%s' type '%s'",
                                         dest_info->address, dest_info->type));

  /***************************************************************************/
  /* Set up the default status                                               */
  /***************************************************************************/
  status.code = OSP_STATUS_SUCCESS;
  status.desc = sca_author_success;

  /***************************************************************************/
  /* Do the authorization checking                                           */
  /***************************************************************************/
  TRC_DETAIL((TB, "Checking authorization"));
  token_info = sca_confirm_authorization(&status, token_list, source_info,
                                                           dest_info, call_id);
  if (token_info != NULL)
  {
    /*************************************************************************/
    /* The token is validated; set the validities we return to those in the  */
    /* token                                                                 */
    /*************************************************************************/
    valid_after = token_info->valid_after;
    valid_until = token_info->valid_until;
  }
  else
  {
    /*************************************************************************/
    /* Something didn't add up; the status code will be set for us, so just  */
    /* set up the validities                                                 */
    /*************************************************************************/
    TRC_ALERT((TB, "Authorization checking failed"));
    valid_after = NULL;
    valid_until = NULL;
  }

  /***************************************************************************/
  /* Send the confirm based on what we've got                                */
  /***************************************************************************/
  TRC_NORMAL((TB, "Sending auth confirm, code %d", status.code));
  rc = osp_ar_auth_confirm(corr, &status, valid_after, valid_until, FALSE);
  if (rc != OSP_SUCCESS)
  {
    /*************************************************************************/
    /* We absolutely should NOT get a bad parameter error; otherwise just    */
    /* trace the result.                                                     */
    /*************************************************************************/
    TRC_ASSERT(((rc != OSP_ERR_BAD_PARAM) && (rc != OSP_ERR_BAD_CORR)),
                         (TB, "Bad parameter errors unexpected from OSP api"));
    TRC_ERROR((TB, "Auth confirm call failed, rc = %d", rc));
  }

  /***************************************************************************/
  /* Free our resources                                                      */
  /***************************************************************************/
  sca_free_token_info(token_info);

  TRC_EXIT();

  return(rc);

} /* sca_auth_ind_callback */

