#ifndef SCACONST_INCLUDED
#define SCACONST_INCLUDED

/**INC+***********************************************************************/
/* Header:    scaconst.h                                                     */
/*                                                                           */
/* Purpose:   Sample Call Authorization and Routing Component                */
/*            Global constants                                               */
/*                                                                           */
/* Copyright (c) 2000, 2001 Data Connection Limited                          */
/* Copyright (c) 2000, 2001 Cisco Systems, Inc. All Rights Reserved.         */
/*                                                                           */
/* All software incorporated in the "OpenOSP Server" (except for those       */
/* portions incorporating third party software specifically identified as    */
/* being licensed under separate license) are licensed under the terms of    */
/* the Cisco Public Source License ver. 1.0 (2000) found in                  */
/* $OSPROOT/license.txt.  Additional copies of the license are available     */
/* from opensource@cisco.com.                                                */
/*                                                                           */
/* Also, before using this software, please see $OSPROOT/x509_ipr.txt for    */
/* information regarding intellectual property rights.                       */
/*                                                                           */
/* $Revision::   1.22               $ $Modtime::   Jan 11 2001 15:56:16   $  */
/*                                                                           */
/**INC-***********************************************************************/

/*****************************************************************************/
/* The base dn of our auth rq directory                                      */
/*****************************************************************************/
#ifdef SCA_DEFINE_VARS
const char *sca_ar_base = "dbname=OpenOSP";
#else
extern const char *sca_ar_base;
#endif

/*****************************************************************************/
/* Lists of attributes we want returned on the ldap search                   */
/*****************************************************************************/
#define SCA_NUM_ATTRS   5
#ifdef SCA_DEFINE_VARS
const char *sca_auth_rq_attrs[SCA_NUM_ATTRS];
#else
extern const char *sca_auth_rq_attrs[SCA_NUM_ATTRS];
#endif

/*****************************************************************************/
/* Component Descriptions                                                    */
/*****************************************************************************/
#ifdef SCA_DEFINE_VARS
AIB_ELEMENT sca_token_info_elements[] =
{
  {
    com_source_info,
    aib_start_processing_address,
    AIB_OCCURS_QUMK,
    offsetof(SCA_TOKEN_INFO, source_info),
    0,
    NULL
  },

  {
    com_source_alt,
    aib_start_processing_list,
    AIB_OCCURS_STAR,
    offsetof(SCA_TOKEN_INFO, source_alt_list),
    0,
    (void *)aib_start_processing_address
  },

  {
    com_dest_info,
    aib_start_processing_address,
    AIB_OCCURS_QUMK,
    offsetof(SCA_TOKEN_INFO, dest_info),
    0,
    NULL
  },

  {
    com_dest_alt,
    aib_start_processing_list,
    AIB_OCCURS_STAR,
    offsetof(SCA_TOKEN_INFO, dest_alt_list),
    0,
    (void *)aib_start_processing_address
  },

  {
    com_call_id,
    aib_start_processing_list,
    AIB_OCCURS_STAR,
    offsetof(SCA_TOKEN_INFO, call_id_list),
    0,
    (void *)aib_start_processing_call_id
  },

  {
    com_valid_after,
    aib_start_processing_simple_element,
    AIB_OCCURS_QUMK,
    offsetof(SCA_TOKEN_INFO, valid_after),
    AIB_DEF_TIME_LEN,
    NULL
  },

  {
    com_valid_until,
    aib_start_processing_simple_element,
    AIB_OCCURS_QUMK,
    offsetof(SCA_TOKEN_INFO, valid_until),
    AIB_DEF_TIME_LEN,
    NULL
  },

  {
    com_trans_id,
    aib_start_processing_simple_element,
    AIB_OCCURS_QUMK,
    offsetof(SCA_TOKEN_INFO, transaction_id),
    AIB_DEF_TRANSACTIONID_LEN,
    NULL
  },

  {
    com_usage,
    aib_start_processing_list,
    AIB_OCCURS_STAR,
    offsetof(SCA_TOKEN_INFO, use_detail_list),
    0,
    (void *)aib_start_processing_usage_detail
  },

  {
    com_authority_url,
    aib_start_processing_list,
    AIB_OCCURS_STAR,
    offsetof(SCA_TOKEN_INFO, auth_url_list),
    0,
    (void *)aib_start_processing_simple_element
  },

  { NULL, NULL, 0, 0, 0, NULL }
};
#else
extern AIB_ELEMENT sca_token_info_elements[];
#endif

/*****************************************************************************/
/* Text strings                                                              */
/*****************************************************************************/
#ifdef SCA_DEFINE_VARS
/*****************************************************************************/
/* Element and attribute names                                               */
/*****************************************************************************/
const char sca_token_info[] = "TokenInfo";
const char sca_e164_type[] = "e164";
const char sca_ip_address[] = "ip_address";
const char sca_almost_out[] = "almost_out";
const char sca_aoor_true[] = "true";
const char sca_aoor_false[] = "false";
const char sca_e164_attr[] = "e164";
const char sca_pricing[] = "pricing";
const char sca_dollars[] = "USD";
const char sca_seconds[] = "s";
const char sca_subscriber[] = "subscriber";
const char sca_transport[] = "transport";
const char sca_prot_type[] = "protocol_type";
const char sca_service_type[] = "service_type";

/*****************************************************************************/
/* Status descriptions                                                       */
/*****************************************************************************/
const char sca_e164_only[] =
                      "Address type not supported; only e164 numbers allowed.";
const char sca_route_not_found[] = "No route found to destination.";
const char sca_unexpected_elem[] = "Token/authentication info unexpected on " \
                                   "authentication-only AuthorizationRequest.";
const char sca_operation_success[] = "Operation successful.";
const char sca_no_operation[] = "No operation specified.";
const char sca_authen_success[] = "Authentication successful.";
const char sca_authen_failed[] = "Authentication failed.";
const char sca_author_success[] = "Authorization successful.";
const char sca_author_failed[] = "Authorization failed.";
const char sca_no_token[] = "Cannot confirm authorization without a token.";
const char sca_token_verify_failed[] = "Could not verify the token signature";
const char sca_unrecognised_token[] = "Unrecognised token format.";
const char sca_invalid_token[] = "Token does not match call information.";
const char sca_pricing_success[] = "Pricing updated successfully.";
const char sca_caps_success[] = "Capabilities updated successfully.";
const char sca_caps_no_transport[] = "Capabilities not updated; no transport" \
                                                          " address provided.";
const char sca_usd_only[] = "Pricing must be in US Dollars per time interval.";
const char sca_bad_request[] = "Bad request: data is invalid.";
#else
extern const char sca_token_info[];
extern const char sca_e164_type[];
extern const char sca_ip_address[];
extern const char sca_almost_out[];
extern const char sca_aoor_true[];
extern const char sca_aoor_false[];
extern const char sca_e164_attr[];
extern const char sca_pricing[];
extern const char sca_dollars[];
extern const char sca_seconds[];
extern const char sca_subscriber[];
extern const char sca_transport[];
extern const char sca_prot_type[];
extern const char sca_service_type[];

extern const char sca_e164_only[];
extern const char sca_route_not_found[];
extern const char sca_unexpected_elem[];
extern const char sca_operation_success[];
extern const char sca_no_operation[];
extern const char sca_authen_success[];
extern const char sca_authen_failed[];
extern const char sca_author_success[];
extern const char sca_author_failed[];
extern const char sca_no_token[];
extern const char sca_token_verify_failed[];
extern const char sca_unrecognised_token[];
extern const char sca_invalid_token[];
extern const char sca_pricing_success[];
extern const char sca_caps_success[];
extern const char sca_caps_no_transport[];
extern const char sca_usd_only[];
extern const char sca_bad_request[];
#endif

/*****************************************************************************/
/* List of services we support                                               */
/*****************************************************************************/
#ifdef SCA_DEFINE_VARS
const char *sca_services_list[SCA_NUM_SERVICES] =
{
  "PricingIndication",
  "AuthorizationRequest",
  "AuthorisationRequest",
  "AuthorizationIndication",
  "AuthorisationIndication",
  "UsageIndication",
  "ReauthorizationRequest",
  "ReauthorisationRequest",
  "SubscriberAuthenticationRequest",
  "CapabilitiesIndication"
};
#else
extern const char *sca_services_list[SCA_NUM_SERVICES];
#endif

#endif
