#ifndef SCADEF_INCLUDED
#define SCADEF_INCLUDED

/**INC+***********************************************************************/
/* Header:    scadef.h                                                       */
/*                                                                           */
/* Purpose:   Sample Call Authorization and Routing Component                */
/*            Internal defines                                               */
/*                                                                           */
/* Copyright (c) 2000, 2001 Data Connection Limited                          */
/* Copyright (c) 2000, 2001 Cisco Systems, Inc. All Rights Reserved.         */
/*                                                                           */
/* All software incorporated in the "OpenOSP Server" (except for those       */
/* portions incorporating third party software specifically identified as    */
/* being licensed under separate license) are licensed under the terms of    */
/* the Cisco Public Source License ver. 1.0 (2000) found in                  */
/* $OSPROOT/license.txt.  Additional copies of the license are available     */
/* from opensource@cisco.com.                                                */
/*                                                                           */
/* Also, before using this software, please see $OSPROOT/x509_ipr.txt for    */
/* information regarding intellectual property rights.                       */
/*                                                                           */
/* $Revision::   1.15               $ $Modtime::   Jan 11 2001 15:56:16   $  */
/*                                                                           */
/**INC-***********************************************************************/

/*****************************************************************************/
/* Length limits on internal strings                                         */
/*****************************************************************************/
#define SCA_MAX_E164_LEN  16
#define SCA_MAX_PRICE_LEN 10
#define SCA_TOKEN_LEN     1024
#define SCA_TRANS_ID_LEN  16

/*****************************************************************************/
/* Validity checking return codes                                            */
/*****************************************************************************/
#define SCA_VALIDITY_OK         1
#define SCA_VALIDITY_EXPIRED    2
#define SCA_VALIDITY_NOT_YET    3
#define SCA_VALIDITY_NOT_VALID  4

/*****************************************************************************/
/* Price status codes returned by sca_check_price_validity()                 */
/*****************************************************************************/
#define SCA_PRICE_CURRENT     1
#define SCA_PRICE_NOT_CURRENT 2
#define SCA_PRICE_EXPIRED     3
#define SCA_PRICE_NOT_FOUND   4

/*****************************************************************************/
/* Number of service names we support                                        */
/*****************************************************************************/
#define SCA_NUM_SERVICES  10

/*****************************************************************************/
/* Spread of authorization validity                                          */
/*****************************************************************************/
#define SCA_CONFIG_AUTH_VALIDITY  "authorization_validity"
#define SCA_DEFAULT_AUTH_VALIDITY 30

/*****************************************************************************/
/* Name of config entry for authorization duration                           */
/*****************************************************************************/
#define SCA_CONFIG_AUTH_DURATION    "authorized_call_duration"

/*****************************************************************************/
/* Name of config entry for LDAP directory that holds routing information    */
/*****************************************************************************/
#define SCA_CONFIG_DIRECTORY        "routing_directory"

/*****************************************************************************/
/* Name of config entry to indicate tokens should be signed                  */
/*****************************************************************************/
#define SCA_CONFIG_SIGN_TOKENS      "token_signing_enabled"

/*****************************************************************************/
/* Name of config entry for identity to use when signing routing information */
/*****************************************************************************/
#define SCA_CONFIG_SIGNING_ID       "token_signing_identity"

/*****************************************************************************/
/* Name of config entry for this server's service URL                        */
/*****************************************************************************/
#define SCA_CONFIG_SERVICE_URL      "service_url"

/*****************************************************************************/
/* Lookup types                                                              */
/*****************************************************************************/
#define SCA_SUBS_AUTHEN_RQ    1
#define SCA_ROUTE_AUTH_RQ     2
#define SCA_ROUTE_REAUTH_RQ   3

/*****************************************************************************/
/* Possible values of oor flag                                               */
/*****************************************************************************/
#define SCA_OOR_MAKE_TRUE     1
#define SCA_OOR_MAKE_FALSE    2
#define SCA_OOR_NO_CHANGE     3

#endif

