
/**MOD+***********************************************************************/
/* Module:    scarrcb.c                                                      */
/*                                                                           */
/* Purpose:   Sample Call Authorization and Routing Component                */
/*            Reauthorization Request processing                             */
/*                                                                           */
/* Copyright (c) 2000, 2001 Data Connection Limited                          */
/* Copyright (c) 2000, 2001 Cisco Systems, Inc. All Rights Reserved.         */
/*                                                                           */
/* All software incorporated in the "OpenOSP Server" (except for those       */
/* portions incorporating third party software specifically identified as    */
/* being licensed under separate license) are licensed under the terms of    */
/* the Cisco Public Source License ver. 1.0 (2000) found in                  */
/* $OSPROOT/license.txt.  Additional copies of the license are available     */
/* from opensource@cisco.com.                                                */
/*                                                                           */
/* Also, before using this software, please see $OSPROOT/x509_ipr.txt for    */
/* information regarding intellectual property rights.                       */
/*                                                                           */
/* $Revision::   1.8                $ $Modtime::   Jan 11 2001 15:56:18   $  */
/*                                                                           */
/**MOD-***********************************************************************/

#include <scaincl.h>

/**PROC+**********************************************************************/
/* Name:      sca_reauth_rq_callback                                         */
/*                                                                           */
/* Purpose:   See openosp.h                                                  */
/*                                                                           */
/* Returns:   See openosp.h                                                  */
/*                                                                           */
/* Params:    See openosp.h                                                  */
/*                                                                           */
/* Operation: Verify the authorization, and issue a response based on the    */
/*            the supplied params and with a revised validity                */
/*                                                                           */
/**PROC-**********************************************************************/

OSP_RC OSPAPI sca_reauth_rq_callback(OSP_CORRELATOR corr,
                                     char *timestamp,
                                     char *role,
                                     OSP_CALL_ID *call_id,
                                     OSP_ADDRESS *source_info,
                                     OSP_ADDRESS *source_alt_list,
                                     OSP_ADDRESS *dest_info,
                                     OSP_ADDRESS *dest_alt_list,
                                     char *transaction_id,
                                     OSP_USAGE_DETAIL *use_detail_list,
                                     OSP_TOKEN *token_list,
                                     OSP_CLIENT_ID *client_id)
{
  /***************************************************************************/
  /* Local Variables                                                         */
  /***************************************************************************/
  OSP_STATUS status;
  OSP_RC rc = OSP_SUCCESS;
  SCA_TOKEN_INFO *token_info;

  TRC_ENTRY("sca_reauth_rq_callback");

  /***************************************************************************/
  /* Check parameters                                                        */
  /***************************************************************************/
  TRC_ASSERT((corr != NULL), (TB, "Passed NULL correlator"));
  if (source_info != NULL)
  {
    TRC_DETAIL((TB, "Reauth Rq for source '%s' type '%s'",
                                     source_info->address, source_info->type));
  }
  if (dest_info != NULL)
  {
    TRC_DETAIL((TB, "RR for destination address '%s' type '%s'",
                                         dest_info->address, dest_info->type));
  }

  /***************************************************************************/
  /* Set up defaults                                                         */
  /***************************************************************************/
  status.code = OSP_STATUS_SUCCESS;
  status.desc = sca_authen_success;

  /***************************************************************************/
  /* Do the authorization checking                                           */
  /***************************************************************************/
  TRC_DETAIL((TB, "Checking authorization"));
  token_info = sca_confirm_authorization(&status, token_list, source_info,
                                                           dest_info, call_id);
  if (token_info == NULL)
  {
    TRC_ALERT((TB, "Authorization checking failed"));
    goto EXIT_LABEL;
  }

  /***************************************************************************/
  /* Now we can go on to create a new routing based on the supplied          */
  /* parameters.  Note that we'll assume we want a single destination        */
  /***************************************************************************/
  TRC_DETAIL((TB, "Doing routing..."));
  rc = sca_do_routing(corr, SCA_ROUTE_REAUTH_RQ, call_id, source_info,
                                          dest_info, dest_alt_list, NULL, "1");
  if (rc != OSP_SUCCESS)
  {
    TRC_ERROR((TB, "routing failed with rc %d", rc));
    status.code = OSP_STATUS_TRANSIENT_PROBLEM_IN_SERVER;
    status.desc = (char *)scm_subs_server_failure;
  }

EXIT_LABEL:
  /***************************************************************************/
  /* If something went wrong, we send an immediate failure                   */
  /***************************************************************************/
  if (status.code != OSP_STATUS_SUCCESS)
  {
    TRC_NORMAL((TB, "Sending auth confirm failure, code %d", status.code));
    rc = osp_ar_reauth_response(corr,
                                &status,
                                transaction_id,
                                NULL,
                                NULL,
                                scm_sign_responses);
    if (rc != OSP_SUCCESS)
    {
      /***********************************************************************/
      /* We absolutely should NOT get a bad parameter error; otherwise just  */
      /* trace the result.                                                   */
      /***********************************************************************/
      TRC_ASSERT(((rc != OSP_ERR_BAD_PARAM) && (rc != OSP_ERR_BAD_CORR)),
                        (TB, "Bad parameter errors unexpected from OSP api"));
      TRC_ERROR((TB, "Reauth response call failed, rc = %d", rc));
    }
  }

  /***************************************************************************/
  /* Free our resources                                                      */
  /***************************************************************************/
  sca_free_token_info(token_info);

  TRC_EXIT();

  return(rc);

} /* sca_reauth_rq_callback */
