#ifndef SCASTRC_INCLUDED
#define SCASTRC_INCLUDED

/**INC+***********************************************************************/
/* Header:    scastrc.h                                                      */
/*                                                                           */
/* Purpose:   Sample Call Authorization and Routing Component                */
/*            Internal structures                                            */
/*                                                                           */
/* Copyright (c) 2000, 2001 Data Connection Limited                          */
/* Copyright (c) 2000, 2001 Cisco Systems, Inc. All Rights Reserved.         */
/*                                                                           */
/* All software incorporated in the "OpenOSP Server" (except for those       */
/* portions incorporating third party software specifically identified as    */
/* being licensed under separate license) are licensed under the terms of    */
/* the Cisco Public Source License ver. 1.0 (2000) found in                  */
/* $OSPROOT/license.txt.  Additional copies of the license are available     */
/* from opensource@cisco.com.                                                */
/*                                                                           */
/* Also, before using this software, please see $OSPROOT/x509_ipr.txt for    */
/* information regarding intellectual property rights.                       */
/*                                                                           */
/* $Revision::   1.8                $ $Modtime::   Jan 11 2001 15:56:20   $  */
/*                                                                           */
/**INC-***********************************************************************/

/**STRUCT+********************************************************************/
/* Structure: SCA_ROUTE_DATA                                                 */
/*                                                                           */
/* Description: Storage for routing search                                   */
/*                                                                           */
/*****************************************************************************/

typedef struct sca_route_data
{
  /***************************************************************************/
  /* Correlator supplied on the SCA callback                                 */
  /***************************************************************************/
  OSP_CORRELATOR corr;

  /***************************************************************************/
  /* Request type                                                            */
  /***************************************************************************/
  int request_type;

  /***************************************************************************/
  /* Other parameters which we need to store from the callback for later use */
  /***************************************************************************/
  /* List of call ids to use                                                 */
  /***************************************************************************/
  OSP_CALL_ID *call_id_list;

  /***************************************************************************/
  /* Address of caller                                                       */
  /***************************************************************************/
  OSP_ADDRESS *source_info;

  /***************************************************************************/
  /* Destination address                                                     */
  /***************************************************************************/
  OSP_ADDRESS *dest_info;

  /***************************************************************************/
  /* List of destination alternates                                          */
  /***************************************************************************/
  OSP_ADDRESS *dest_alt_list;

  /***************************************************************************/
  /* Maximum number of destination addresses to return                       */
  /***************************************************************************/
  int max_dests;

} SCA_ROUTE_DATA, * PSCA_ROUTE_DATA;

/**STRUCT-********************************************************************/

/**STRUCT+********************************************************************/
/* Structure: SCA_ROUTE_OPTION                                               */
/*                                                                           */
/* Description: Contains routing information returned by the LDAP search     */
/*                                                                           */
/*****************************************************************************/

typedef struct sca_route_option
{
  /***************************************************************************/
  /* List managment                                                          */
  /***************************************************************************/
  LIST_ENTRY links;

  /***************************************************************************/
  /* Destination address                                                     */
  /***************************************************************************/
  char *ip_address;

  /***************************************************************************/
  /* Almost out of resources flag                                            */
  /***************************************************************************/
  int almost_out;

  /***************************************************************************/
  /* Matching phone number prefix                                            */
  /***************************************************************************/
  char matching_number[SCA_MAX_E164_LEN];

  /***************************************************************************/
  /* Length of the matching number (used in sorting)                         */
  /***************************************************************************/
  unsigned int best_match;

  /***************************************************************************/
  /* Price (in US Dollars per minute)                                        */
  /***************************************************************************/
  char price[SCA_MAX_PRICE_LEN];

  /***************************************************************************/
  /* Price as a number (used in matching)                                    */
  /***************************************************************************/
  unsigned int best_price;

} SCA_ROUTE_OPTION;

/**STRUCT-********************************************************************/

/**STRUCT+********************************************************************/
/* Structure: SCA_TOKEN_INFO                                                 */
/*                                                                           */
/* Description: Structure to hold the content of an OSP Authorization Token  */
/*                                                                           */
/*****************************************************************************/

typedef struct sca_token_info
{
  /***************************************************************************/
  /* Elements common to all parsed data structures                           */
  /***************************************************************************/
  AIB_DATA common;

  /***************************************************************************/
  /* Source address                                                          */
  /***************************************************************************/
  OSP_ADDRESS *source_info;

  /***************************************************************************/
  /* List of further information about source                                */
  /***************************************************************************/
  SLIST_ROOT source_alt_list;

  /***************************************************************************/
  /* Destination address                                                     */
  /***************************************************************************/
  OSP_ADDRESS *dest_info;

  /***************************************************************************/
  /* List of further information about destination                           */
  /***************************************************************************/
  SLIST_ROOT dest_alt_list;

  /***************************************************************************/
  /* List of supplied call ids                                               */
  /***************************************************************************/
  SLIST_ROOT call_id_list;

  /***************************************************************************/
  /* Time token is valid from                                                */
  /***************************************************************************/
  char *valid_after;

  /***************************************************************************/
  /* Time token is valid until                                               */
  /***************************************************************************/
  char *valid_until;

  /***************************************************************************/
  /* Transaction id associtated with the token                               */
  /***************************************************************************/
  char *transaction_id;

  /***************************************************************************/
  /* List of usage details                                                   */
  /***************************************************************************/
  SLIST_ROOT use_detail_list;

  /***************************************************************************/
  /* List of authority URLs                                                  */
  /***************************************************************************/
  SLIST_ROOT auth_url_list;

} SCA_TOKEN_INFO;

/**STRUCT-********************************************************************/

/**STRUCT+********************************************************************/
/* Structure: SCA_PRICE_DATA                                                 */
/*                                                                           */
/* Description: Storage for pricing updates                                  */
/*                                                                           */
/*****************************************************************************/

typedef struct sca_price_data
{
  /***************************************************************************/
  /* Correlator supplied on the SCA callback                                 */
  /***************************************************************************/
  OSP_CORRELATOR corr;

  /***************************************************************************/
  /* Other parameters which we need need to store from the callback for      */
  /* later use                                                               */
  /***************************************************************************/
  /* Source information - the transport address of the gateway               */
  /***************************************************************************/
  OSP_ADDRESS *source_info;

  /***************************************************************************/
  /* Destination address - the e164 prefix for the new pricing               */
  /***************************************************************************/
  OSP_ADDRESS *dest_info;

  /***************************************************************************/
  /* The new price                                                           */
  /***************************************************************************/
  char *price;

  /***************************************************************************/
  /* Time pricing is valid from                                              */
  /***************************************************************************/
  char *valid_from;

  /***************************************************************************/
  /* Time pricing valid until                                                */
  /***************************************************************************/
  char *valid_until;

} SCA_PRICE_DATA;

/**STRUCT-********************************************************************/

/**STRUCT+********************************************************************/
/* Structure: SCA_CAPS_DATA                                                  */
/*                                                                           */
/* Description: Storage for caps updates                                     */
/*                                                                           */
/*****************************************************************************/

typedef struct sca_caps_data
{
  /***************************************************************************/
  /* Correlator supplied on the SCA callback                                 */
  /***************************************************************************/
  OSP_CORRELATOR corr;

  /***************************************************************************/
  /* Other parameters which we need need to store from the callback for      */
  /* later use                                                               */
  /***************************************************************************/
  /* Client Version information                                              */
  /***************************************************************************/
  char *osp_version;

  /***************************************************************************/
  /* OOR setting                                                             */
  /***************************************************************************/
  int oor_flag;

  /***************************************************************************/
  /* Supported protocols list                                                */
  /***************************************************************************/
  OSP_PROTOCOL_TYPE *prot_list;

  /***************************************************************************/
  /* Count of protocols in prot_list                                         */
  /***************************************************************************/
  int prot_count;

  /***************************************************************************/
  /* List of client caps for which service information should be returned    */
  /***************************************************************************/
  OSP_CAPS *caps_list;

} SCA_CAPS_DATA;

/**STRUCT-********************************************************************/

#endif
