
/**MOD+***********************************************************************/
/* Module:    scmmain.c                                                      */
/*                                                                           */
/* Purpose:   Sample Application Common management routines                  */
/*                                                                           */
/* Copyright (c) 2000, 2001 Data Connection Limited                          */
/* Copyright (c) 2000, 2001 Cisco Systems, Inc. All Rights Reserved.         */
/*                                                                           */
/* All software incorporated in the "OpenOSP Server" (except for those       */
/* portions incorporating third party software specifically identified as    */
/* being licensed under separate license) are licensed under the terms of    */
/* the Cisco Public Source License ver. 1.0 (2000) found in                  */
/* $OSPROOT/license.txt.  Additional copies of the license are available     */
/* from opensource@cisco.com.                                                */
/*                                                                           */
/* Also, before using this software, please see $OSPROOT/x509_ipr.txt for    */
/* information regarding intellectual property rights.                       */
/*                                                                           */
/* $Revision::   1.2                $ $Modtime::   Jan 11 2001 15:56:22   $  */
/*                                                                           */
/**MOD-***********************************************************************/

#include <scmincl.h>

/**PROC+**********************************************************************/
/* Name:      scm_init                                                       */
/*                                                                           */
/* Purpose:   Do common initialization for sample OpenOSP application        */
/*                                                                           */
/* Returns:   OSP_SUCCESS         - initialized successfully                 */
/*                                                                           */
/* Params:    IN     config       - Pointer to OpenSSL config hash table.    */
/*                                                                           */
/**PROC-**********************************************************************/

OSP_RC scm_init(LHASH *config)
{
  /***************************************************************************/
  /* Local variables.                                                        */
  /***************************************************************************/
  OSP_RC rc = OSP_SUCCESS;
  char *value;

  TRC_ENTRY("scm_init");

  /***************************************************************************/
  /* Check configuration parameter.                                          */
  /***************************************************************************/
  TRC_ASSERT((config != NULL), (TB, "NULL configuration parameter"));

  TRC_DETAIL((TB, "Common initialization for sample application"));

  /***************************************************************************/
  /* Read the OSP config file to discover whether to S/MIME sign responses.  */
  /* The only affirmative value is "yes".                                    */
  /***************************************************************************/
  value = CONF_get_string(config,
                          SCM_CONFIG_SECTION,
                          SCM_CONFIG_SIGN_RESPONSES);

  if ((value != NULL) && (!strcmp(value, SCM_CONFIG_DO_SIGN_RESPONSES)))
  {
    TRC_NORMAL((TB, "S/MIME sign responses"));
    scm_sign_responses = 1;
  }
  else
  {
    TRC_NORMAL((TB, "Don't S/MIME sign responses"));
    scm_sign_responses = 0;
  }

EXIT_LABEL:
  TRC_EXIT();
  return(rc);

} /* scm_init */

/**PROC+**********************************************************************/
/* Name:      scm_term                                                       */
/*                                                                           */
/* Purpose:   Do common termination for sample OpenOSP application           */
/*                                                                           */
/* Returns:   nothing                                                        */
/*                                                                           */
/* Params:    none                                                           */
/*                                                                           */
/**PROC-**********************************************************************/

void scm_term()
{
  TRC_ENTRY("scm_term");

  TRC_DETAIL((TB, "Common termination for sample application"));

  /***************************************************************************/
  /* Nothing actually to do here; function only provided for symmetry.       */
  /***************************************************************************/

EXIT_LABEL:
  TRC_EXIT();
  return;

} /* scm_term */
