
#ifndef AIBCONST_INCLUDED
#define AIBCONST_INCLUDED

/**INC+***********************************************************************/
/* Header:    aibconst.h                                                     */
/*                                                                           */
/* Purpose:   Common AIB - global constants                                  */
/*                                                                           */
/* Copyright (c) 2000, 2001 Data Connection Limited                          */
/* Copyright (c) 2000, 2001 Cisco Systems, Inc. All Rights Reserved.         */
/*                                                                           */
/* All software incorporated in the "OpenOSP Server" (except for those       */
/* portions incorporating third party software specifically identified as    */
/* being licensed under separate license) are licensed under the terms of    */
/* the Cisco Public Source License ver. 1.0 (2000) found in                  */
/* $OSPROOT/license.txt.  Additional copies of the license are available     */
/* from opensource@cisco.com.                                                */
/*                                                                           */
/* Also, before using this software, please see $OSPROOT/x509_ipr.txt for    */
/* information regarding intellectual property rights.                       */
/*                                                                           */
/* $Revision::   1.8                $ $Modtime::   Jan 11 2001 15:56:24   $  */
/*                                                                           */
/**INC-***********************************************************************/

#ifdef AIB_DEFINE_VARS
AIB_ELEMENT aib_service_elements[] = {
  {
    com_bandwidth,
    aib_start_processing_simple_element,
    AIB_OCCURS_QUMK,
    offsetof(AIB_SERVICE_INFO, osp_service.bandwidth),
    AIB_DEF_BANDWIDTH_LEN,
    NULL
  },

  {
    com_service_type,
    aib_start_processing_simple_element,
    AIB_OCCURS_QUMK,
    offsetof(AIB_SERVICE_INFO, osp_service.service_type),
    AIB_DEF_SERVICE_TYPE_LEN,
    NULL
  },

  {
    com_protocol_type,
    aib_start_processing_list,
    AIB_OCCURS_STAR,
    offsetof(AIB_SERVICE_INFO, prot_type_list),
    0,
    (void *)aib_start_processing_protocol_type
  },

  { NULL, NULL, 0, 0, 0, NULL }
};

AIB_ELEMENT aib_termination_cause_elements[] = {
  { com_term_code,            aib_start_processing_simple_element,
    AIB_OCCURS_ONCE,          offsetof(OSP_TERM_CAUSE, term_code),
                              AIB_DEF_TERMCODE_LEN,
                              NULL },

  { com_term_desc,            aib_start_processing_simple_element,
    AIB_OCCURS_QUMK,          offsetof(OSP_TERM_CAUSE, term_desc),
                              AIB_DEF_TERMDESC_LEN,
                              NULL },

  { NULL, NULL, 0, 0, 0, NULL }
};

AIB_ELEMENT aib_statistics_pf_elements[] = {
  { com_packets,              aib_start_processing_simple_element,
    AIB_OCCURS_ONCE,          offsetof(OSP_USAGE_STATISTICS_PF, packets),
                              AIB_DEF_PACKETS_LEN,
                              NULL },

  { com_fraction,             aib_start_processing_simple_element,
    AIB_OCCURS_ONCE,          offsetof(OSP_USAGE_STATISTICS_PF, fraction),
                              AIB_DEF_FRACTION_LEN,
                              NULL },

  { NULL, NULL, 0, 0, 0, NULL }
};

AIB_ELEMENT aib_statistics_mmvs_elements[] = {
  { com_minimum,              aib_start_processing_simple_element,
    AIB_OCCURS_ONCE,          offsetof(OSP_USAGE_STATISTICS_MMVS, minimum),
                              AIB_DEF_MINIMUM_LEN,
                              NULL },

  { com_mean,                 aib_start_processing_simple_element,
    AIB_OCCURS_ONCE,          offsetof(OSP_USAGE_STATISTICS_MMVS, mean),
                              AIB_DEF_MEAN_LEN,
                              NULL },

  { com_variance,             aib_start_processing_simple_element,
    AIB_OCCURS_ONCE,          offsetof(OSP_USAGE_STATISTICS_MMVS, variance),
                              AIB_DEF_VARIANCE_LEN,
                              NULL },

  { com_samples,              aib_start_processing_simple_element,
    AIB_OCCURS_ONCE,          offsetof(OSP_USAGE_STATISTICS_MMVS, samples),
                              AIB_DEF_SAMPLES_LEN,
                              NULL },

  { NULL, NULL, 0, 0, 0, NULL }
};

AIB_ELEMENT aib_statistics_elements[] = {
  { com_loss_sent,            aib_start_processing_substructure,
    AIB_OCCURS_QUMK,          offsetof(OSP_USAGE_STATISTICS, loss_sent),
                              sizeof(OSP_USAGE_STATISTICS_PF),
                              aib_statistics_pf_elements },

  { com_loss_received,        aib_start_processing_substructure,
    AIB_OCCURS_QUMK,          offsetof(OSP_USAGE_STATISTICS, loss_received),
                              sizeof(OSP_USAGE_STATISTICS_PF),
                              aib_statistics_pf_elements },

  { com_one_way_delay,        aib_start_processing_substructure,
    AIB_OCCURS_QUMK,          offsetof(OSP_USAGE_STATISTICS, one_way_delay),
                              sizeof(OSP_USAGE_STATISTICS_MMVS),
                              aib_statistics_mmvs_elements },

  { com_round_trip_delay,     aib_start_processing_substructure,
    AIB_OCCURS_QUMK,          offsetof(OSP_USAGE_STATISTICS, round_trip_delay),
                              sizeof(OSP_USAGE_STATISTICS_MMVS),
                              aib_statistics_mmvs_elements },

  { NULL, NULL, 0, 0, 0, NULL }
};

AIB_ELEMENT aib_usage_detail_elements[] = {
  { com_service,              aib_start_processing_substructure,
    AIB_OCCURS_QUMK,          offsetof(OSP_USAGE_DETAIL, service),
                              sizeof(OSP_SERVICE_INFO),
                              aib_service_elements },

  { com_amount,               aib_start_processing_simple_element,
    AIB_OCCURS_ONCE,          offsetof(OSP_USAGE_DETAIL, amount),
                              AIB_DEF_AMOUNT_LEN,
                              NULL },

  { com_increment,            aib_start_processing_simple_element,
    AIB_OCCURS_ONCE,          offsetof(OSP_USAGE_DETAIL, increment),
                              AIB_DEF_INCREMENT_LEN,
                              NULL },

  { com_service,              aib_start_processing_substructure,
    AIB_OCCURS_QUMK,          offsetof(OSP_USAGE_DETAIL, service),
                              sizeof(OSP_SERVICE_INFO),
                              aib_service_elements },

  { com_unit,                 aib_start_processing_simple_element,
    AIB_OCCURS_ONCE,          offsetof(OSP_USAGE_DETAIL, unit),
                              AIB_DEF_UNIT_LEN,
                              NULL },

  { com_start_time,           aib_start_processing_simple_element,
    AIB_OCCURS_QUMK,          offsetof(OSP_USAGE_DETAIL, start_time),
                              AIB_DEF_TIME_LEN,
                              NULL },

  { com_end_time,             aib_start_processing_simple_element,
    AIB_OCCURS_QUMK,          offsetof(OSP_USAGE_DETAIL, end_time),
                              AIB_DEF_TIME_LEN,
                              NULL },

  { com_term_cause,           aib_start_processing_substructure,
    AIB_OCCURS_QUMK,          offsetof(OSP_USAGE_DETAIL, term_cause),
                              sizeof(OSP_TERM_CAUSE),
                              aib_termination_cause_elements },

  { com_statistics,           aib_start_processing_substructure,
    AIB_OCCURS_QUMK,          offsetof(OSP_USAGE_DETAIL, statistics),
                              sizeof(OSP_USAGE_STATISTICS),
                              aib_statistics_elements },

  { NULL, NULL, 0, 0, 0, NULL }
};

AIB_ELEMENT aib_data_rate_elements[] =
{
  {
    com_num_channels,
    aib_start_processing_simple_element,
    AIB_OCCURS_QUMK,
    offsetof(OSP_DATA_RATE, num_channels),
    AIB_DEF_CHANNELS_LEN,
    NULL
  },

  {
    com_bandwidth,
    aib_start_processing_simple_element,
    AIB_OCCURS_ONCE,
    offsetof(OSP_DATA_RATE, bandwidth),
    AIB_DEF_BANDWIDTH_LEN,
    NULL
  },

  { NULL, NULL, 0, 0, 0, NULL }
};

AIB_ELEMENT aib_supp_protocol_elements[] =
{
  {
    com_protocol_type,
    aib_start_processing_protocol_type,
    AIB_OCCURS_QUMK,
    offsetof(AIB_SUPP_PROTOCOL, supp_prot.prot_type),
    sizeof(OSP_PROTOCOL_TYPE),
    NULL
  },

  {
    com_data_rate,
    aib_start_processing_list,
    AIB_OCCURS_STAR,
    offsetof(AIB_SUPP_PROTOCOL, data_rate_list),
    0,
    (void *)aib_start_processing_data_rate
  },

  { NULL, NULL, 0, 0, 0, NULL }
};

AIB_ELEMENT aib_resource_elements[] =
{
  {
    com_data_rate,
    aib_start_processing_substructure,
    AIB_OCCURS_QUMK,
    offsetof(AIB_RESOURCE, resource.data_rate),
    sizeof(OSP_DATA_RATE),
    aib_data_rate_elements
  },

  {
    com_almost_out,
    aib_start_processing_simple_element,
    AIB_OCCURS_QUMK,
    offsetof(AIB_RESOURCE, resource.almost_out),
    AIB_DEF_ALMOST_OUT_LEN,
    NULL
  },

  {
    com_supp_protocol,
    aib_start_processing_list,
    AIB_OCCURS_STAR,
    offsetof(AIB_RESOURCE, supp_protocol_list),
    0,
    (void *)aib_start_processing_supp_prot
  },

  { NULL, NULL, 0, 0, 0, NULL }
};

#else
extern AIB_ELEMENT aib_service_elements[];
extern AIB_ELEMENT aib_termination_cause_elements[];
extern AIB_ELEMENT aib_statistics_pf_elements[];
extern AIB_ELEMENT aib_statistics_mmvs_elements[];
extern AIB_ELEMENT aib_statistics_elements[];
extern AIB_ELEMENT aib_usage_detail_elements[];
extern AIB_ELEMENT aib_data_rate_elements[];
extern AIB_ELEMENT aib_supp_protocol_elements[];
extern AIB_ELEMENT aib_resource_elements[];
#endif

#ifdef OSP_DATA_COVERAGE
/*****************************************************************************/
/* List of all defined AIB_ELEMENT lists, used by data coverage routines.    */
/*****************************************************************************/
#ifdef AIB_DEFINE_VARS
AIB_COVERAGE_STRUCTURE aib_coverage_structures[] = {
  { "service",                   &(aib_service_elements[0]) },
  { "termination_cause",         &(aib_termination_cause_elements[0]) },
  { "statistics_pf",             &(aib_statistics_pf_elements[0]) },
  { "statistics_mmvs",           &(aib_statistics_mmvs_elements[0]) },
  { "statistics",                &(aib_statistics_elements[0]) },
  { "usage_detail",              &(aib_usage_detail_elements[0]) },
  { "usage_indication",          &(use_usage_indication_elements[0]) },
  { "authorization_request",     &(car_authorization_request_elements[0]) },
  { "authorization_indication",  &(car_authorization_indication_elements[0]) },
  { "reauthorization_request",   &(car_reauthorization_request_elements[0]) },
  { "pricing_indication",        &(car_pricing_indication_elements[0]) },
  { "data_rate",                 &(aib_data_rate_elements[0]) },
  { "supported_protocol",        &(aib_supp_protocol_elements[0]) },
  { "resource",                  &(aib_resource_elements[0]) },
  { "capabilities_indication",   &(car_caps_indication_elements[0]) },
  { "subs_authentication_rq",    &(car_subs_authen_request_elements[0]) },
  { NULL,                        NULL }
};
#else
extern AIB_COVERAGE_STRUCTURE aib_coverage_structures[];
#endif
#endif

#endif
