#ifndef AIBSTRC_INCLUDED
#define AIBSTRC_INCLUDED

/**INC+***********************************************************************/
/* Header:    aibstrc.h                                                      */
/*                                                                           */
/* Purpose:   AIB Common structures                                          */
/*                                                                           */
/* Copyright (c) 2000, 2001 Data Connection Limited                          */
/* Copyright (c) 2000, 2001 Cisco Systems, Inc. All Rights Reserved.         */
/*                                                                           */
/* All software incorporated in the "OpenOSP Server" (except for those       */
/* portions incorporating third party software specifically identified as    */
/* being licensed under separate license) are licensed under the terms of    */
/* the Cisco Public Source License ver. 1.0 (2000) found in                  */
/* $OSPROOT/license.txt.  Additional copies of the license are available     */
/* from opensource@cisco.com.                                                */
/*                                                                           */
/* Also, before using this software, please see $OSPROOT/x509_ipr.txt for    */
/* information regarding intellectual property rights.                       */
/*                                                                           */
/* $Revision::   1.5                $ $Modtime::   Jan 11 2001 15:56:26   $  */
/*                                                                           */
/**INC-***********************************************************************/

#ifdef OSP_DATA_COVERAGE
/**STRUCT+********************************************************************/
/* Structure: AIB_REGISTERED_STRUCTURE                                       */
/*                                                                           */
/* Description: A structure describing a structure that has been registered  */
/*              for data coverage                                            */
/*                                                                           */
/*****************************************************************************/

typedef struct aib_registered_structure
{
  /***************************************************************************/
  /* Pointer to next entry in list.                                          */
  /***************************************************************************/
  struct aib_registered_structure *next;

  /***************************************************************************/
  /* The AIB_ELEMENT array describing the structure type of which this is an */
  /* instance.                                                               */
  /***************************************************************************/
  AIB_ELEMENT *elements;

  /***************************************************************************/
  /* The registered structure's address.                                     */
  /***************************************************************************/
  void *address;

} AIB_REGISTERED_STRUCTURE;

/**STRUCT-********************************************************************/
#endif

/**STRUCT+********************************************************************/
/* Structure: AIB_RESOURCE                                                   */
/*                                                                           */
/* Description: Structure used in parsing a resource                         */
/*                                                                           */
/*****************************************************************************/

typedef struct aib_resource
{
  /***************************************************************************/
  /* The actual OSP_RESOURCE underlying the AIB version                      */
  /***************************************************************************/
  OSP_RESOURCE resource;

  /***************************************************************************/
  /* List structure used in building supported protocols list                */
  /***************************************************************************/
  SLIST_ROOT supp_protocol_list;

} AIB_RESOURCE;

/**STRUCT-********************************************************************/

/**STRUCT+********************************************************************/
/* Structure: AIB_SUPP_PROTOCOL                                              */
/*                                                                           */
/* Description: Structure used in parsing a supported protocol               */
/*                                                                           */
/*****************************************************************************/

typedef struct aib_supp_protocol
{
  /***************************************************************************/
  /* The actual OSP_SUPP_PROTOCOL underlying the AIB version                 */
  /***************************************************************************/
  OSP_SUPP_PROTOCOL supp_prot;

  /***************************************************************************/
  /* List used to build data rates list                                      */
  /***************************************************************************/
  SLIST_ROOT data_rate_list;

} AIB_SUPP_PROTOCOL;

/**STRUCT-********************************************************************/

/**STRUCT+********************************************************************/
/* Structure: AIB_SERVICE_INFO                                               */
/*                                                                           */
/* Description: Structure used in parsing a service info                     */
/*                                                                           */
/*****************************************************************************/

typedef struct aib_service_info
{
  /***************************************************************************/
  /* The actual OSP_SERVICE_INFO underlying the AIB version                  */
  /***************************************************************************/
  OSP_SERVICE_INFO osp_service;

  /***************************************************************************/
  /* List used to build data rates list                                      */
  /***************************************************************************/
  SLIST_ROOT prot_type_list;

} AIB_SERVICE_INFO;

/**STRUCT-********************************************************************/

#endif
