
#ifndef CARCONST_INCLUDED
#define CARCONST_INCLUDED

/**INC+***********************************************************************/
/* Header:    carconst.h                                                     */
/*                                                                           */
/* Purpose:   Call Authorization and Routing (CAR) Application Interface     */
/*            Layer (AIL) global constants                                   */
/*                                                                           */
/* Copyright (c) 2000, 2001 Data Connection Limited                          */
/* Copyright (c) 2000, 2001 Cisco Systems, Inc. All Rights Reserved.         */
/*                                                                           */
/* All software incorporated in the "OpenOSP Server" (except for those       */
/* portions incorporating third party software specifically identified as    */
/* being licensed under separate license) are licensed under the terms of    */
/* the Cisco Public Source License ver. 1.0 (2000) found in                  */
/* $OSPROOT/license.txt.  Additional copies of the license are available     */
/* from opensource@cisco.com.                                                */
/*                                                                           */
/* Also, before using this software, please see $OSPROOT/x509_ipr.txt for    */
/* information regarding intellectual property rights.                       */
/*                                                                           */
/* $Revision::   1.12               $ $Modtime::   Jan 11 2001 15:56:28   $  */
/*                                                                           */
/**INC-***********************************************************************/

/*****************************************************************************/
/* Error descriptions                                                        */
/*****************************************************************************/
#ifdef CAR_DEFINE_VARS
const char car_critical_element_not_supported[] =
                                              "Critical element not supported";
const char car_parsing_unsuccessful[] = "Parsing unsuccessful";
const char car_service_not_available[] = "Service not available";
const char car_transient_problem_in_server[] = "Transient problem in server";
const char car_num_call_ids_wrong[] =
                             "Max dests does not match the number of call ids";
#else
extern const char car_critical_element_not_supported[];
extern const char car_parsing_unsuccessful[];
extern const char car_service_not_available[];
extern const char car_transient_problem_in_server[];
extern const char car_num_call_ids_wrong[];
#endif

/*****************************************************************************/
/* Component Descriptions                                                    */
/*****************************************************************************/
#ifdef CAR_DEFINE_VARS

AIB_ELEMENT car_authorization_request_elements[] =
{
  {
    com_timestamp,
    aib_start_processing_simple_element,
    AIB_OCCURS_ONCE,
    offsetof(CAR_PARSED_AR, timestamp),
    AIB_DEF_TIME_LEN,
    NULL
  },

  {
    com_call_id,
    aib_start_processing_list,
    AIB_OCCURS_PLUS,
    offsetof(CAR_PARSED_AR, call_id_list),
    sizeof(OSP_CALL_ID),
    (void *)aib_start_processing_call_id
  },

  {
    com_source_info,
    aib_start_processing_address,
    AIB_OCCURS_ONCE,
    offsetof(CAR_PARSED_AR, source_info),
    sizeof(OSP_ADDRESS),
    NULL
  },

  {
    com_source_alt,
    aib_start_processing_list,
    AIB_OCCURS_STAR,
    offsetof(CAR_PARSED_AR, source_alt_list),
    0,
    (void *)aib_start_processing_address
  },

  {
    com_dest_info,
    aib_start_processing_address,
    AIB_OCCURS_ONCE,
    offsetof(CAR_PARSED_AR, dest_info),
    sizeof(OSP_ADDRESS),
    NULL
  },

  {
    com_dest_alt,
    aib_start_processing_list,
    AIB_OCCURS_STAR,
    offsetof(CAR_PARSED_AR, dest_alt_list),
    0,
    (void *)aib_start_processing_address
  },

  {
    com_service,
    aib_start_processing_substructure,
    AIB_OCCURS_ONCE,
    offsetof(CAR_PARSED_AR, service),
    sizeof(AIB_SERVICE_INFO),
    aib_service_elements
  },

  {
    com_max_dests,
    aib_start_processing_simple_element,
    AIB_OCCURS_ONCE,
    offsetof(CAR_PARSED_AR, max_dests),
    AIB_DEF_MAX_DESTS_LEN,
    NULL
  },

  {
    com_token,
    aib_start_processing_list,
    AIB_OCCURS_STAR,
    offsetof(CAR_PARSED_AR, token_list),
    0,
    (void *)aib_start_processing_token
  },

  {
    com_subs_authen,
    aib_start_processing_list,
    AIB_OCCURS_STAR,
    offsetof(CAR_PARSED_AR, subs_auth_list),
    0,
    (void *)aib_start_processing_subs_auth
  },

  { NULL, NULL, 0, 0, 0, NULL }
};

AIB_ELEMENT car_authorization_indication_elements[] =
{
  {
    com_timestamp,
    aib_start_processing_simple_element,
    AIB_OCCURS_ONCE,
    offsetof(CAR_PARSED_AI, timestamp),
    AIB_DEF_TIME_LEN,
    NULL
  },

  {
    com_role,
    aib_start_processing_simple_element,
    AIB_OCCURS_ONCE,
    offsetof(CAR_PARSED_AI, role),
    AIB_DEF_ROLE_LEN,
    NULL
  },

  {
    com_call_id,
    aib_start_processing_call_id,
    AIB_OCCURS_ONCE,
    offsetof(CAR_PARSED_AI, call_id),
    sizeof(OSP_CALL_ID),
    NULL
  },

  {
    com_source_info,
    aib_start_processing_address,
    AIB_OCCURS_ONCE,
    offsetof(CAR_PARSED_AI, source_info),
    sizeof(OSP_ADDRESS),
    NULL
  },

  {
    com_source_alt,
    aib_start_processing_list,
    AIB_OCCURS_STAR,
    offsetof(CAR_PARSED_AI, source_alt_list),
    0,
    (void *)aib_start_processing_address
  },

  {
    com_dest_info,
    aib_start_processing_address,
    AIB_OCCURS_ONCE,
    offsetof(CAR_PARSED_AI, dest_info),
    sizeof(OSP_ADDRESS),
    NULL
  },

  {
    com_dest_alt,
    aib_start_processing_list,
    AIB_OCCURS_STAR,
    offsetof(CAR_PARSED_AI, dest_alt_list),
    0,
    (void *)aib_start_processing_address
  },

  {
    com_service,
    aib_start_processing_substructure,
    AIB_OCCURS_ONCE,
    offsetof(CAR_PARSED_AI, service),
    sizeof(OSP_SERVICE_INFO),
    aib_service_elements
  },

  {
    com_token,
    aib_start_processing_list,
    AIB_OCCURS_STAR,
    offsetof(CAR_PARSED_AI, token_list),
    0,
    (void *)aib_start_processing_token
  },

  { NULL, NULL, 0, 0, 0, NULL }
};

AIB_ELEMENT car_reauthorization_request_elements[] =
{
  {
    com_timestamp,
    aib_start_processing_simple_element,
    AIB_OCCURS_ONCE,
    offsetof(CAR_PARSED_RR, timestamp),
    AIB_DEF_TIME_LEN,
    NULL
  },

  {
    com_role,
    aib_start_processing_simple_element,
    AIB_OCCURS_ONCE,
    offsetof(CAR_PARSED_AI, role),
    AIB_DEF_ROLE_LEN,
    NULL
  },

  {
    com_call_id,
    aib_start_processing_call_id,
    AIB_OCCURS_ONCE,
    offsetof(CAR_PARSED_AI, call_id),
    sizeof(OSP_CALL_ID),
    NULL
  },

  {
    com_source_info,
    aib_start_processing_address,
    AIB_OCCURS_QUMK,
    offsetof(CAR_PARSED_RR, source_info),
    0,
    NULL
  },

  {
    com_source_alt,
    aib_start_processing_list,
    AIB_OCCURS_STAR,
    offsetof(CAR_PARSED_RR, source_alt_list),
    0,
    (void *)aib_start_processing_address
  },

  {
    com_dest_info,
    aib_start_processing_address,
    AIB_OCCURS_QUMK,
    offsetof(CAR_PARSED_RR, dest_info),
    0,
    NULL
  },

  {
    com_dest_alt,
    aib_start_processing_list,
    AIB_OCCURS_STAR,
    offsetof(CAR_PARSED_RR, dest_alt_list),
    0,
    (void *)aib_start_processing_address
  },

  {
    com_trans_id,
    aib_start_processing_simple_element,
    AIB_OCCURS_ONCE,
    offsetof(CAR_PARSED_RR, transaction_id),
    AIB_DEF_TRANSACTIONID_LEN,
    NULL
  },

  {
    com_usage,
    aib_start_processing_list,
    AIB_OCCURS_STAR,
    offsetof(CAR_PARSED_RR, use_detail_list),
    0,
    (void *)aib_start_processing_usage_detail
  },

  {
    com_token,
    aib_start_processing_list,
    AIB_OCCURS_STAR,
    offsetof(CAR_PARSED_RR, token_list),
    0,
    (void *)aib_start_processing_token
  },

  { NULL, NULL, 0, 0, 0, NULL }
};

AIB_ELEMENT car_pricing_indication_elements[] =
{
  {
    com_timestamp,
    aib_start_processing_simple_element,
    AIB_OCCURS_ONCE,
    offsetof(CAR_PARSED_PI, timestamp),
    AIB_DEF_TIME_LEN,
    NULL
  },

  {
    com_source_info,
    aib_start_processing_address,
    AIB_OCCURS_ONCE,
    offsetof(CAR_PARSED_PI, source_info),
    sizeof(OSP_ADDRESS),
    NULL
  },

  {
    com_dest_info,
    aib_start_processing_address,
    AIB_OCCURS_ONCE,
    offsetof(CAR_PARSED_PI, dest_info),
    sizeof(OSP_ADDRESS),
    NULL
  },

  {
    com_currency,
    aib_start_processing_simple_element,
    AIB_OCCURS_ONCE,
    offsetof(CAR_PARSED_PI, currency),
    AIB_DEF_AMOUNT_LEN,
    NULL
  },

  {
    com_amount,
    aib_start_processing_simple_element,
    AIB_OCCURS_ONCE,
    offsetof(CAR_PARSED_PI, amount),
    AIB_DEF_AMOUNT_LEN,
    NULL
  },

  {
    com_increment,
    aib_start_processing_simple_element,
    AIB_OCCURS_ONCE,
    offsetof(CAR_PARSED_PI, increment),
    AIB_DEF_INCREMENT_LEN,
    NULL
  },

  {
    com_unit,
    aib_start_processing_simple_element,
    AIB_OCCURS_ONCE,
    offsetof(CAR_PARSED_PI, unit),
    AIB_DEF_UNIT_LEN,
    NULL
  },

  {
    com_service,
    aib_start_processing_substructure,
    AIB_OCCURS_ONCE,
    offsetof(CAR_PARSED_PI, service),
    sizeof(OSP_SERVICE_INFO),
    aib_service_elements
  },

  {
    com_valid_after,
    aib_start_processing_simple_element,
    AIB_OCCURS_ONCE,
    offsetof(CAR_PARSED_PI, valid_after),
    AIB_DEF_TIME_LEN,
    NULL
  },

  {
    com_valid_until,
    aib_start_processing_simple_element,
    AIB_OCCURS_ONCE,
    offsetof(CAR_PARSED_PI, valid_until),
    AIB_DEF_TIME_LEN,
    NULL
  },

  { NULL, NULL, 0, 0, 0, NULL }
};

AIB_ELEMENT car_caps_indication_elements[] =
{
  {
    com_dev_info,
    aib_start_processing_list,
    AIB_OCCURS_STAR,
    offsetof(CAR_PARSED_CI, dev_info_list),
    0,
    (void *)aib_start_processing_dev_info
  },

  {
    com_osp_version,
    aib_start_processing_simple_element,
    AIB_OCCURS_ONCE,
    offsetof(CAR_PARSED_CI, version),
    AIB_DEF_VERSION_LEN,
    NULL
  },

  {
    com_osp_capability,
    aib_start_processing_list,
    AIB_OCCURS_STAR,
    offsetof(CAR_PARSED_CI, caps_list),
    0,
    (void *)aib_start_processing_capability
  },

  {
    com_resources,
    aib_start_processing_list,
    AIB_OCCURS_STAR,
    offsetof(CAR_PARSED_CI, resource_list),
    0,
    (void *)aib_start_processing_resource
  },

  { NULL, NULL, 0, 0, 0, NULL }
};

AIB_ELEMENT car_subs_authen_request_elements[] =
{
  {
    com_timestamp,
    aib_start_processing_simple_element,
    AIB_OCCURS_ONCE,
    offsetof(CAR_PARSED_SAR, timestamp),
    AIB_DEF_TIME_LEN,
    NULL
  },

  {
    com_source_info,
    aib_start_processing_address,
    AIB_OCCURS_ONCE,
    offsetof(CAR_PARSED_SAR, source_info),
    sizeof(OSP_ADDRESS),
    NULL
  },

  {
    com_source_alt,
    aib_start_processing_list,
    AIB_OCCURS_STAR,
    offsetof(CAR_PARSED_SAR, source_alt_list),
    0,
    (void *)aib_start_processing_address
  },

  {
    com_dest_info,
    aib_start_processing_address,
    AIB_OCCURS_QUMK,
    offsetof(CAR_PARSED_SAR, dest_info),
    sizeof(OSP_ADDRESS),
    NULL
  },

  {
    com_service,
    aib_start_processing_substructure,
    AIB_OCCURS_QUMK,
    offsetof(CAR_PARSED_SAR, service),
    sizeof(OSP_SERVICE_INFO),
    aib_service_elements
  },

  {
    com_subs_authen,
    aib_start_processing_subs_auth,
    AIB_OCCURS_QUMK,
    offsetof(CAR_PARSED_SAR, subs_authen_info),
    sizeof(OSP_AUTH_INFO),
    NULL
  },

  { NULL, NULL, 0, 0, 0, NULL }
};

#else

extern AIB_ELEMENT car_authorization_request_elements[];
extern AIB_ELEMENT car_authorization_indication_elements[];
extern AIB_ELEMENT car_reauthorization_request_elements[];
extern AIB_ELEMENT car_pricing_indication_elements[];
extern AIB_ELEMENT car_caps_indication_elements[];
extern AIB_ELEMENT car_subs_authen_request_elements[];
#endif

#endif
