#ifndef CARSTRC_INCLUDED
#define CARSTRC_INCLUDED

/**INC+***********************************************************************/
/* Header:    carstrc.h                                                      */
/*                                                                           */
/* Purpose:   Call Authorization and Routing (CAR) Application Interface     */
/*            Layer (AIL) structure definitions                              */
/*                                                                           */
/* Copyright (c) 2000, 2001 Data Connection Limited                          */
/* Copyright (c) 2000, 2001 Cisco Systems, Inc. All Rights Reserved.         */
/*                                                                           */
/* All software incorporated in the "OpenOSP Server" (except for those       */
/* portions incorporating third party software specifically identified as    */
/* being licensed under separate license) are licensed under the terms of    */
/* the Cisco Public Source License ver. 1.0 (2000) found in                  */
/* $OSPROOT/license.txt.  Additional copies of the license are available     */
/* from opensource@cisco.com.                                                */
/*                                                                           */
/* Also, before using this software, please see $OSPROOT/x509_ipr.txt for    */
/* information regarding intellectual property rights.                       */
/*                                                                           */
/* $Revision::   1.11               $ $Modtime::   Jan 11 2001 15:56:30   $  */
/*                                                                           */
/**INC-***********************************************************************/

/**STRUCT+********************************************************************/
/* Structure:   CAR_PARSED_AR                                                */
/*                                                                           */
/* Description: CAR AIB Parsed Authorization Request                         */
/*                                                                           */
/* Allocated:   car_start_processing(); stored in trans->aib_state           */
/*                                                                           */
/* Freed:       car_free_ar_mem(), called from car_end_auth_rq_handler() or  */
/*              car_cancel_processing()                                      */
/*                                                                           */
/*****************************************************************************/

typedef struct car_parsed_ar
{
  /***************************************************************************/
  /* Elements common to all parsed data structures                           */
  /***************************************************************************/
  AIB_DATA common;

  /***************************************************************************/
  /* Timestamp                                                               */
  /***************************************************************************/
  char *timestamp;

  /***************************************************************************/
  /* List of supplied call ids                                               */
  /***************************************************************************/
  SLIST_ROOT call_id_list;

  /***************************************************************************/
  /* Source address                                                          */
  /***************************************************************************/
  OSP_ADDRESS *source_info;

  /***************************************************************************/
  /* List of further information about source                                */
  /***************************************************************************/
  SLIST_ROOT source_alt_list;

  /***************************************************************************/
  /* Destination address                                                     */
  /***************************************************************************/
  OSP_ADDRESS *dest_info;

  /***************************************************************************/
  /* List of further information about destination                           */
  /***************************************************************************/
  SLIST_ROOT dest_alt_list;

  /***************************************************************************/
  /* Service information                                                     */
  /***************************************************************************/
  AIB_SERVICE_INFO *service;

  /***************************************************************************/
  /* Maximum number of destination address to be returned                    */
  /***************************************************************************/
  char *max_dests;

  /***************************************************************************/
  /* List of authorization tokens supplied                                   */
  /***************************************************************************/
  SLIST_ROOT token_list;

  /***************************************************************************/
  /* List containing subscriber authentication information                   */
  /***************************************************************************/
  SLIST_ROOT subs_auth_list;

} CAR_PARSED_AR;
/**STRUCT-********************************************************************/

/**STRUCT+********************************************************************/
/* Structure:   CAR_PARSED_AI                                                */
/*                                                                           */
/* Description: CAR AIB Parsed Authorization Indication                      */
/*                                                                           */
/* Allocated:   car_start_processing(); stored in trans->aib_state           */
/*                                                                           */
/* Freed:       car_free_ai_mem(), called from car_end_auth_ind_handler() or */
/*              car_cancel_processing()                                      */
/*                                                                           */
/*****************************************************************************/

typedef struct car_parsed_ai
{
  /***************************************************************************/
  /* Elements common to all parsed data structures                           */
  /***************************************************************************/
  AIB_DATA common;

  /***************************************************************************/
  /* Timestamp                                                               */
  /***************************************************************************/
  char *timestamp;

  /***************************************************************************/
  /* Role in call                                                            */
  /***************************************************************************/
  char *role;

  /***************************************************************************/
  /* Supplied call id                                                        */
  /***************************************************************************/
  OSP_CALL_ID *call_id;

  /***************************************************************************/
  /* Source address                                                          */
  /***************************************************************************/
  OSP_ADDRESS *source_info;

  /***************************************************************************/
  /* List of further information about source                                */
  /***************************************************************************/
  SLIST_ROOT source_alt_list;

  /***************************************************************************/
  /* Destination address                                                     */
  /***************************************************************************/
  OSP_ADDRESS *dest_info;

  /***************************************************************************/
  /* List of further information about destination                           */
  /***************************************************************************/
  SLIST_ROOT dest_alt_list;

  /***************************************************************************/
  /* Service information                                                     */
  /***************************************************************************/
  OSP_SERVICE_INFO *service;

  /***************************************************************************/
  /* List of authorization tokens supplied                                   */
  /***************************************************************************/
  SLIST_ROOT token_list;

} CAR_PARSED_AI;
/**STRUCT-********************************************************************/

/**STRUCT+********************************************************************/
/* Structure:   CAR_PARSED_RR                                                */
/*                                                                           */
/* Description: CAR AIB Parsed Reauthorization Request                       */
/*                                                                           */
/* Allocated:   car_start_processing(); stored in trans->aib_state           */
/*                                                                           */
/* Freed:       car_free_rr_mem(), called from car_end_reauth_rq_handler()   */
/*              or car_cancel_processing()                                   */
/*                                                                           */
/*****************************************************************************/

typedef struct car_parsed_rr
{
  /***************************************************************************/
  /* Elements common to all parsed data structures                           */
  /***************************************************************************/
  AIB_DATA common;

  /***************************************************************************/
  /* Timestamp                                                               */
  /***************************************************************************/
  char *timestamp;

  /***************************************************************************/
  /* Role in call                                                            */
  /***************************************************************************/
  char *role;

  /***************************************************************************/
  /* Supplied call id                                                        */
  /***************************************************************************/
  OSP_CALL_ID *call_id;

  /***************************************************************************/
  /* Source address                                                          */
  /***************************************************************************/
  OSP_ADDRESS *source_info;

  /***************************************************************************/
  /* List of further information about source                                */
  /***************************************************************************/
  SLIST_ROOT source_alt_list;

  /***************************************************************************/
  /* Destination address                                                     */
  /***************************************************************************/
  OSP_ADDRESS *dest_info;

  /***************************************************************************/
  /* List of further information about destination                           */
  /***************************************************************************/
  SLIST_ROOT dest_alt_list;

  /***************************************************************************/
  /* Supplied transaction ID                                                 */
  /***************************************************************************/
  char *transaction_id;

  /***************************************************************************/
  /* Supplied list of usage details                                          */
  /***************************************************************************/
  SLIST_ROOT use_detail_list;

  /***************************************************************************/
  /* List of authorization tokens supplied                                   */
  /***************************************************************************/
  SLIST_ROOT token_list;

} CAR_PARSED_RR;
/**STRUCT-********************************************************************/

/**STRUCT+********************************************************************/
/* Structure:   CAR_PARSED_PI                                                */
/*                                                                           */
/* Description: CAR AIB Parsed Pricing Indication                            */
/*                                                                           */
/* Allocated:   car_start_processing(); stored in trans->aib_state           */
/*                                                                           */
/* Freed:       car_free_pi_mem(), called from                               */
/*              car_pricing_ind_end_elem_handler() or car_cancel_processing()*/
/*                                                                           */
/*****************************************************************************/

typedef struct car_parsed_pi
{
  /***************************************************************************/
  /* Elements common to all parsed data structures                           */
  /***************************************************************************/
  AIB_DATA common;

  /***************************************************************************/
  /* Timestamp                                                               */
  /***************************************************************************/
  char *timestamp;

  /***************************************************************************/
  /* Source address                                                          */
  /***************************************************************************/
  OSP_ADDRESS *source_info;

  /***************************************************************************/
  /* Destination address                                                     */
  /***************************************************************************/
  OSP_ADDRESS *dest_info;

  /***************************************************************************/
  /* Currency                                                                */
  /***************************************************************************/
  char *currency;

  /***************************************************************************/
  /* Amount of servce                                                        */
  /***************************************************************************/
  char *amount;

  /***************************************************************************/
  /* Increment of time                                                       */
  /***************************************************************************/
  char *increment;

  /***************************************************************************/
  /* Unit of time                                                            */
  /***************************************************************************/
  char *unit;

  /***************************************************************************/
  /* Service for which this pricing is intended                              */
  /***************************************************************************/
  OSP_SERVICE_INFO *service;

  /***************************************************************************/
  /* Time after which price is valid                                         */
  /***************************************************************************/
  char *valid_after;

  /***************************************************************************/
  /* Time price is valid until                                               */
  /***************************************************************************/
  char *valid_until;

} CAR_PARSED_PI;
/**STRUCT-********************************************************************/

/**STRUCT+********************************************************************/
/* Structure:   CAR_PARSED_CI                                                */
/*                                                                           */
/* Description: CAR AIB Parsed Capabilities Indication                       */
/*                                                                           */
/* Allocated:   car_start_processing(); stored in trans->aib_state           */
/*                                                                           */
/* Freed:       car_free_ci_mem(), called from                               */
/*              car_pricing_ind_end_elem_handler() or car_cancel_processing()*/
/*                                                                           */
/*****************************************************************************/

typedef struct car_parsed_ci
{
  /***************************************************************************/
  /* Elements common to all parsed data structures                           */
  /***************************************************************************/
  AIB_DATA common;

  /***************************************************************************/
  /* List of device information structures                                   */
  /***************************************************************************/
  SLIST_ROOT dev_info_list;

  /***************************************************************************/
  /* OSP Version                                                             */
  /***************************************************************************/
  char *version;

  /***************************************************************************/
  /* List of device capabilities                                             */
  /***************************************************************************/
  SLIST_ROOT caps_list;

  /***************************************************************************/
  /* List of device resources                                                */
  /***************************************************************************/
  SLIST_ROOT resource_list;

} CAR_PARSED_CI;
/**STRUCT-********************************************************************/

/**STRUCT+********************************************************************/
/* Structure:   CAR_PARSED_SAR                                               */
/*                                                                           */
/* Description: CAR AIB Parsed Subscriber Authentication Request             */
/*                                                                           */
/* Allocated:   car_start_processing(); stored in trans->aib_state           */
/*                                                                           */
/* Freed:       car_free_sar_mem(), called from                              */
/*              car_caps_ind_end_elem_handler() or car_cancel_processing()   */
/*                                                                           */
/*****************************************************************************/

typedef struct car_parsed_sar
{
  /***************************************************************************/
  /* Elements common to all parsed data structures                           */
  /***************************************************************************/
  AIB_DATA common;

  /***************************************************************************/
  /* Timestamp                                                               */
  /***************************************************************************/
  char *timestamp;

  /***************************************************************************/
  /* Source address                                                          */
  /***************************************************************************/
  OSP_ADDRESS *source_info;

  /***************************************************************************/
  /* List of further information about source                                */
  /***************************************************************************/
  SLIST_ROOT source_alt_list;

  /***************************************************************************/
  /* Destination address                                                     */
  /***************************************************************************/
  OSP_ADDRESS *dest_info;

  /***************************************************************************/
  /* Service information                                                     */
  /***************************************************************************/
  OSP_SERVICE_INFO *service;

  /***************************************************************************/
  /* Cisco extension: subscriber authentication information                  */
  /***************************************************************************/
  OSP_AUTH_INFO *subs_authen_info;

} CAR_PARSED_SAR;
/**STRUCT-********************************************************************/

#endif
