#ifndef CCMCONST_INCLUDED
#define CCMCONST_INCLUDED

/**INC+***********************************************************************/
/* Header:    ccmconst.h                                                     */
/*                                                                           */
/* Purpose:   CCM constants                                                  */
/*                                                                           */
/* Copyright (c) 2000, 2001 Data Connection Limited                          */
/* Copyright (c) 2000, 2001 Cisco Systems, Inc. All Rights Reserved.         */
/*                                                                           */
/* All software incorporated in the "OpenOSP Server" (except for those       */
/* portions incorporating third party software specifically identified as    */
/* being licensed under separate license) are licensed under the terms of    */
/* the Cisco Public Source License ver. 1.0 (2000) found in                  */
/* $OSPROOT/license.txt.  Additional copies of the license are available     */
/* from opensource@cisco.com.                                                */
/*                                                                           */
/* Also, before using this software, please see $OSPROOT/x509_ipr.txt for    */
/* information regarding intellectual property rights.                       */
/*                                                                           */
/* $Revision::   1.19               $ $Modtime::   Jan 11 2001 15:56:30   $  */
/*                                                                           */
/**INC-***********************************************************************/

/*****************************************************************************/
/* The config file group and entries for identity information.               */
/*****************************************************************************/
#ifdef CCM_DEFINE_VARS
const char ccm_cfg_group_identities[] = "identities";
const char ccm_cfg_identities_id_dn[] = "identity_%d_dn";
const char ccm_cfg_identities_id_pkey[] = "identity_%d_privkey";
#else
extern const char ccm_cfg_group_identities[];
extern const char ccm_cfg_identities_id_dn[];
extern const char ccm_cfg_identities_id_pkey[];
#endif

/*****************************************************************************/
/* General config file entries.                                              */
/*****************************************************************************/
#ifdef CCM_DEFINE_VARS
const char ccm_cfg_rand_file[] = "random_seed_file";
#else
extern const char ccm_cfg_rand_file[];
#endif

/*****************************************************************************/
/* The config file entries for CA functionality.                             */
/*****************************************************************************/
#ifdef CCM_DEFINE_VARS
const char ccm_cfg_ca_serial_file[] = "ca_serial_file";
const char ccm_cfg_ca_duration[] = "ca_valid_duration";
const char ccm_cfg_ca_rsa_sign_digest[] = "ca_rsa_signing_digest";
const char ccm_cfg_ca_subject_suffix[] = "ca_subject_name_suffix";
#else
extern const char ccm_cfg_ca_serial_file[];
extern const char ccm_cfg_ca_duration[];
extern const char ccm_cfg_ca_rsa_sign_digest[];
extern const char ccm_cfg_ca_subject_suffix[];
#endif

/*****************************************************************************/
/* The config file group containing the X.509 v3 extension policy.           */
/*****************************************************************************/
#ifdef CCM_DEFINE_VARS
const char ccm_cfg_group_extensions[] = "ca_cert_extensions";
#else
extern const char ccm_cfg_group_extensions[];
#endif

/*****************************************************************************/
/* The config file entries for the LDAP certificate repository.              */
/*****************************************************************************/
#ifdef CCM_DEFINE_VARS
const char ccm_cfg_ldap_host[] = "ca_directory";
const char ccm_cfg_ldap_user[] = "ca_directory_user";
const char ccm_cfg_ldap_user_dflt[] = "";
const char ccm_cfg_ldap_password[] = "ca_directory_password";
const char ccm_cfg_ldap_password_dflt[] = "";
const char ccm_cfg_ldap_dn_suffix[] = "ca_directory_dn_suffix";
#else
extern const char ccm_cfg_ldap_host[];
extern const char ccm_cfg_ldap_user[];
extern const char ccm_cfg_ldap_user_dflt[];
extern const char ccm_cfg_ldap_password[];
extern const char ccm_cfg_ldap_password_dflt[];
extern const char ccm_cfg_ldap_dn_suffix[];
#endif

/*****************************************************************************/
/* The config file entries for SSL functionality.                            */
/*****************************************************************************/
#ifdef CCM_DEFINE_VARS
const char ccm_cfg_ssl_identity[] = "ssl_identity";
const char ccm_cfg_ssl_verify_client[] = "ssl_verify_client";
const char ccm_cfg_ssl_cipher_list[] = "ssl_cipher_list";
const char ccm_cfg_ssl_cipher_list_dflt[] = "DEFAULT";
const char ccm_cfg_vfy_no[] = "no";
const char ccm_cfg_vfy_yes[] = "yes";
const char ccm_cfg_vfy_once[] = "once";
const char ccm_cfg_vfy_require[] = "require";
const char ccm_cfg_vfy_separators[] = ":, \t";
#else
extern const char ccm_cfg_ssl_identity[];
extern const char ccm_cfg_ssl_verify_client[];
extern const char ccm_cfg_ssl_cipher_list[];
extern const char ccm_cfg_ssl_cipher_list_dflt[];
extern const char ccm_cfg_vfy_no[];
extern const char ccm_cfg_vfy_yes[];
extern const char ccm_cfg_vfy_once[];
extern const char ccm_cfg_vfy_require[];
extern const char ccm_cfg_vfy_separators[];
#endif

/*****************************************************************************/
/* The object attributes that we use to store and retrieve certificates and  */
/* CRLs in the LDAP repository.                                              */
/*****************************************************************************/
#ifdef CCM_DEFINE_VARS
const char ccm_ldap_attr_usercert[] = "userCertificate";
const char ccm_ldap_attr_cacert[] = "cACertificate";
const char ccm_ldap_attr_crl[] = "certificateRevocationList";
const char ccm_ldap_attr_objclass[] = "objectClass";
const char ccm_ldap_attr_top[] = "top";
const char ccm_ldap_attr_sauser[] = "strongAuthenticationUser";
const char ccm_ldap_attr_ca[] = "certificationAuthority";
#else
extern const char ccm_ldap_attr_usercert[];
extern const char ccm_ldap_attr_cacert[];
extern const char ccm_ldap_attr_crl[];
extern const char ccm_ldap_attr_objclass[];
extern const char ccm_ldap_attr_top[];
extern const char ccm_ldap_attr_sauser[];
extern const char ccm_ldap_attr_ca[];
#endif

/*****************************************************************************/
/* The search filter that we use to find certs/CRLs via LDAP.                */
/*****************************************************************************/
#ifdef CCM_DEFINE_VARS
const char ccm_ldap_search_cert_filter[] =
                                     "(|(objectclass=strongAuthenticationUser)"
                                       "(objectclass=certificationAuthority))";
const char ccm_ldap_search_crl_filter[] =
                                        "(objectclass=certificationAuthority)";
#else
extern const char ccm_ldap_search_cert_filter[];
extern const char ccm_ldap_search_crl_filter[];
#endif

/*****************************************************************************/
/* Hex conversion table.                                                     */
/*****************************************************************************/
#ifdef CCM_DEFINE_VARS
const char ccm_hex[]="0123456789ABCDEF";
#else
extern const char ccm_hex[];
#endif

/*****************************************************************************/
/* Pre-generated 512-bit Diffie-Hellman parameters.                          */
/*****************************************************************************/
#ifdef CCM_DEFINE_VARS
const unsigned char ccm_dh512_p[64]={
                               0x91, 0x32, 0xF8, 0x3A, 0x93, 0x1F, 0x3F, 0xB4,
                               0x15, 0xB5, 0x4C, 0xB3, 0x78, 0xA6, 0x63, 0xEA,
                               0x19, 0xF4, 0x06, 0xB7, 0x90, 0x6D, 0x03, 0xE0,
                               0xA0, 0x08, 0x32, 0xCE, 0x76, 0xAB, 0x85, 0xF5,
                               0x55, 0xFD, 0x52, 0xE1, 0x3A, 0x23, 0xF4, 0x9E,
                               0x3D, 0x4B, 0x7E, 0x4C, 0x27, 0x0F, 0xB6, 0x00,
                               0x74, 0x56, 0x5B, 0x2D, 0x28, 0x41, 0xF3, 0x15,
                               0x15, 0x99, 0x5C, 0x6B, 0xE5, 0x7C, 0xFD, 0x1B};
const unsigned char ccm_dh512_g[1]={0x02};
#else
extern const unsigned char ccm_dh512_p[64];
extern const unsigned char ccm_dh512_g[1];
#endif

/*****************************************************************************/
/* Pre-generated 1024-bit Diffie-Hellman parameters.                         */
/*****************************************************************************/
#ifdef CCM_DEFINE_VARS
const unsigned char ccm_dh1024_p[128]={
                               0xBC, 0xBA, 0x3D, 0x1C, 0x7E, 0x8E, 0xE3, 0xFE,
                               0x7E, 0x98, 0x0D, 0x54, 0xC4, 0x82, 0x90, 0xE3,
                               0xE7, 0x93, 0x78, 0xA5, 0x09, 0x93, 0x6C, 0xD9,
                               0xFC, 0x97, 0xB9, 0xC2, 0x27, 0x99, 0x0A, 0xC1,
                               0x6F, 0x27, 0x63, 0x2B, 0xD1, 0x59, 0xAD, 0x9D,
                               0x46, 0x53, 0x2F, 0x57, 0x9C, 0x8E, 0x7A, 0x4F,
                               0x79, 0xE7, 0x4F, 0x61, 0x3A, 0x7C, 0xA4, 0x37,
                               0xBC, 0x83, 0xED, 0x66, 0xA6, 0xB8, 0x80, 0x0C,
                               0x83, 0x78, 0xD5, 0x80, 0x74, 0xB2, 0x96, 0x9F,
                               0xBC, 0x2D, 0x9A, 0x95, 0x8A, 0x11, 0x26, 0x69,
                               0x41, 0xB6, 0xDD, 0x4C, 0xE7, 0x8F, 0x86, 0x9D,
                               0xC7, 0x1E, 0xC0, 0xBF, 0xA4, 0xBF, 0x53, 0x11,
                               0x5E, 0x50, 0x9B, 0x29, 0x5D, 0x92, 0x18, 0x7C,
                               0x4A, 0x09, 0x9D, 0x1B, 0x58, 0xA2, 0x2B, 0xBD,
                               0x38, 0xEF, 0x12, 0x31, 0xFA, 0xE9, 0x4D, 0xDE,
                               0x72, 0x1B, 0x5F, 0x02, 0xA8, 0xA6, 0x05, 0x93};
const unsigned char ccm_dh1024_g[1]={0x02};
#else
extern const unsigned char ccm_dh1024_p[128];
extern const unsigned char ccm_dh1024_g[1];
#endif

/*****************************************************************************/
/* Prompt for passphrase                                                     */
/*****************************************************************************/
#ifdef CCM_DEFINE_VARS
const char ccm_pp_prompt[] =
                 "Please enter the private key passphrase for identity '%s': ";
#else
extern const char ccm_pp_prompt[];
#endif

#endif

