#ifndef CCMEXTRN_INCLUDED
#define CCMEXTRN_INCLUDED

/**INC+***********************************************************************/
/* Header:    ccmextrn.h                                                     */
/*                                                                           */
/* Purpose:   CCM API header                                                 */
/*                                                                           */
/* Copyright (c) 2000, 2001 Data Connection Limited                          */
/* Copyright (c) 2000, 2001 Cisco Systems, Inc. All Rights Reserved.         */
/*                                                                           */
/* All software incorporated in the "OpenOSP Server" (except for those       */
/* portions incorporating third party software specifically identified as    */
/* being licensed under separate license) are licensed under the terms of    */
/* the Cisco Public Source License ver. 1.0 (2000) found in                  */
/* $OSPROOT/license.txt.  Additional copies of the license are available     */
/* from opensource@cisco.com.                                                */
/*                                                                           */
/* Also, before using this software, please see $OSPROOT/x509_ipr.txt for    */
/* information regarding intellectual property rights.                       */
/*                                                                           */
/* $Revision::   1.8                $ $Modtime::   Jan 11 2001 15:56:30   $  */
/*                                                                           */
/**INC-***********************************************************************/

/*****************************************************************************/
/* Flags for use with ccm_pkcs7_xxx().                                       */
/*****************************************************************************/
#define CCM_FLAG_SELF_SIGNED 0x01
#define CCM_FLAG_PKCS7_D     0x02
#define CCM_FLAG_MASK        0x03

/*****************************************************************************/
/* Digests for use with ccm_pkcs7_sign().                                    */
/*****************************************************************************/
#define CCM_DIGEST_SHA1                1
#define CCM_DIGEST_MD5                 2
#define CCM_DIGEST_DEFAULT             CCM_DIGEST_SHA1

/*****************************************************************************/
/* Ciphers for use with ccm_pkcs7_encrypt().                                 */
/*****************************************************************************/
#define CCM_CIPHER_DES_CBC             1
#define CCM_CIPHER_RC2_CBC             2
#define CCM_CIPHER_DEFAULT             CCM_CIPHER_DES_CBC

/*****************************************************************************/
/* Callback definitions.                                                     */
/*****************************************************************************/
typedef void (*PCCM_NEW_CERT_CALLBACK)
                       (void *,
                        char *,
                        unsigned int);

typedef int (*PCCM_SSL_VERIFY_CALLBACK)
                       (SSL *,
                        char *,
                        unsigned int,
                        unsigned int,
                        unsigned int);

/*****************************************************************************/
/* Procedure declarations.                                                   */
/*****************************************************************************/
OSP_RC ccm_init(SSL_CTX *,
                PCCM_SSL_VERIFY_CALLBACK);

OSP_RC ccm_term(void);

OSP_RC ccm_pkcs7_sign(char *,
                      char *,
                      unsigned int,
                      unsigned int,
                      STACK_OF(X509_ATTRIBUTE) *,
                      unsigned int,
                      char **,
                      unsigned int *);

OSP_RC ccm_pkcs7_verify(char *,
                        unsigned int,
                        char *,
                        unsigned int,
                        unsigned int,
                        STACK_OF(X509_ATTRIBUTE) **,
                        char **,
                        unsigned int *,
                        char **,
                        unsigned int *);

OSP_RC ccm_pkcs7_encrypt(char *,
                         char *,
                         unsigned int,
                         char *,
                         unsigned int,
                         unsigned int,
                         char **,
                         unsigned int *);

OSP_RC ccm_pkcs7_decrypt(char *,
                         unsigned int,
                         char *,
                         char **,
                         unsigned int *);

OSP_RC ccm_get_cert(char *,
                    char **,
                    unsigned int *);

OSP_RC ccm_get_cert_chain(char *,
                          OSP_CERTIFICATE **);

void ccm_free_cert_chain(OSP_CERTIFICATE *);

OSP_RC ccm_request_new_cert(char *,
                            unsigned int ,
                            char *,
                            char *,
                            PCCM_NEW_CERT_CALLBACK,
                            void *);

void ccm_random(void *,
                int);

#endif

