#ifndef CCMFUNC_INCLUDED
#define CCMFUNC_INCLUDED

/**INC+***********************************************************************/
/* Header:    ccmfunc.h                                                      */
/*                                                                           */
/* Purpose:   CCM internal function prototypes                               */
/*                                                                           */
/* Copyright (c) 2000, 2001 Data Connection Limited                          */
/* Copyright (c) 2000, 2001 Cisco Systems, Inc. All Rights Reserved.         */
/*                                                                           */
/* All software incorporated in the "OpenOSP Server" (except for those       */
/* portions incorporating third party software specifically identified as    */
/* being licensed under separate license) are licensed under the terms of    */
/* the Cisco Public Source License ver. 1.0 (2000) found in                  */
/* $OSPROOT/license.txt.  Additional copies of the license are available     */
/* from opensource@cisco.com.                                                */
/*                                                                           */
/* Also, before using this software, please see $OSPROOT/x509_ipr.txt for    */
/* information regarding intellectual property rights.                       */
/*                                                                           */
/* $Revision::   1.13               $ $Modtime::   Jan 11 2001 15:56:32   $  */
/*                                                                           */
/**INC-***********************************************************************/

/*****************************************************************************/
/* ccmca.c                                                                   */
/*****************************************************************************/
void *ccm_cert_request_thread(void *);

/*****************************************************************************/
/* ccmcaut.c                                                                 */
/*****************************************************************************/
OSP_RC ccm_expand_subject_name(X509_NAME *,
                               X509_NAME **);

void ccm_free_req_context(CCM_NEW_CERT_CONTEXT *);

OSP_RC ccm_read_serial_number(void);

OSP_RC ccm_update_serial_number(void);

void ccm_show_fingerprints(X509 *);

/*****************************************************************************/
/* ccmconf.c                                                                 */
/*****************************************************************************/
OSP_RC ccm_load_config(void);

static int ccm_passphrase_cb(char *, int, int, void *);

/*****************************************************************************/
/* ccmldap.c                                                                 */
/*****************************************************************************/
int ccm_lookup_ldap_by_subject(X509_LOOKUP *,
                               int,
                               X509_NAME *,
                               X509_OBJECT *);

OSP_RC ccm_store_new_cert(X509 *,
                          char *,
                          unsigned int,
                          char *);

OSP_RC ccm_add_ldap_dn_suffix(char **);

OSP_RC ccm_ldap_bind(void);

/*****************************************************************************/
/* ccmpk7ut.c                                                                */
/*****************************************************************************/
PKCS7 *ccm_int_pkcs7_sign(X509 *,
                          EVP_PKEY *,
                          STACK_OF(X509) *,
                          BIO *,
                          int,
                          unsigned int,
                          STACK_OF(X509_ATTRIBUTE) *);

OSP_RC ccm_int_pkcs7_verify(PKCS7 *,
                            STACK_OF(X509) *,
                            X509_STORE *,
                            BIO *,
                            BIO *,
                            int,
                            unsigned int);

/*****************************************************************************/
/* ccmrand.c                                                                 */
/*****************************************************************************/
OSP_RC ccm_read_random_state(void);

OSP_RC ccm_write_random_state(void);

/*****************************************************************************/
/* ccmutil.c                                                                 */
/*****************************************************************************/
OSP_RC ccm_get_auth_attribs(PKCS7 *,
                            STACK_OF(X509_ATTRIBUTE) **);

OSP_RC ccm_rfc2253_to_x509_name(char *,
                                X509_NAME **);

OSP_RC ccm_x509_name_to_rfc2253(X509_NAME *,
                                char **);

int ccm_must_escape(char,
                    unsigned int,
                    unsigned int);

OSP_RC ccm_get_priv_key(char *,
                        EVP_PKEY **);

OSP_RC ccm_int_get_cert(char *,
                        X509 **);

OSP_RC ccm_int_get_cert_x509_name(X509_NAME *,
                                  X509 **);

RSA *ccm_temp_rsa_callback(SSL *, int, int);

DH *ccm_temp_dh_callback(SSL *, int, int);

DH *ccm_get_dh512(void);

DH *ccm_get_dh1024(void);

/*****************************************************************************/
/* ccmvfy.c                                                                  */
/*****************************************************************************/
int ccm_verify_callback(int,
                        X509_STORE_CTX *);

int ccm_check_crl(X509_STORE_CTX *,
                  char *,
                  char *);

OSP_RC ccm_get_valid_crl(X509_STORE_CTX *,
                         X509_NAME *,
                         char *,
                         X509_CRL **);

#endif

