#ifndef COMEXTRN_INCLUDED
#define COMEXTRN_INCLUDED

/**INC+***********************************************************************/
/* Header:    comextrn.h                                                     */
/*                                                                           */
/* Purpose:   OSP Common Functions, External API                             */
/*                                                                           */
/* Copyright (c) 2000, 2001 Data Connection Limited                          */
/* Copyright (c) 2000, 2001 Cisco Systems, Inc. All Rights Reserved.         */
/*                                                                           */
/* All software incorporated in the "OpenOSP Server" (except for those       */
/* portions incorporating third party software specifically identified as    */
/* being licensed under separate license) are licensed under the terms of    */
/* the Cisco Public Source License ver. 1.0 (2000) found in                  */
/* $OSPROOT/license.txt.  Additional copies of the license are available     */
/* from opensource@cisco.com.                                                */
/*                                                                           */
/* Also, before using this software, please see $OSPROOT/x509_ipr.txt for    */
/* information regarding intellectual property rights.                       */
/*                                                                           */
/* $Revision::   1.29               $ $Modtime::   Jan 11 2001 15:56:34   $  */
/*                                                                           */
/**INC-***********************************************************************/

#ifdef OSP_MEASURE_TRANSACTION
#include <smtime.h>
#endif

/*****************************************************************************/
/* Constants                                                                 */
/*****************************************************************************/
#define TRUE  1
#define FALSE 0

/*****************************************************************************/
/* Space required for cached timestamp time string.                          */
/*****************************************************************************/
#define COM_DEF_TIME_LEN 21

/*****************************************************************************/
/* Value indicating that renegotiate time has not yet been set               */
/*****************************************************************************/
#define COM_RENEGOTIATE_NOT_SET      (time_t)-2

/*****************************************************************************/
/* Byte type for pointer arithmetic                                          */
/*****************************************************************************/
typedef unsigned char OSP_BYTE;

/*****************************************************************************/
/* Config file section name for stack configuration                          */
/*****************************************************************************/
#define OSP_CONFIG_STACK_GROUP   "openosp"

/*****************************************************************************/
/* Copy a string, first getting the memory in which to place the copy        */
/*****************************************************************************/
#define OSP_STRDUP(dest, string)                                           \
  dest = (char *)rm_get_mem(strlen(string) + 1);                           \
  if (dest)                                                                \
  {                                                                        \
    TRC_DETAIL((TB, "strdup got len %d at %p", strlen(string) + 1, dest)); \
    strcpy((dest), (string));                                              \
  }

/*****************************************************************************/
/* Wrapper for (v)snprintf which may not be available on all platforms       */
/*****************************************************************************/
#define OSP_SNPRINTF(params)     snprintf params
#define OSP_VSNPRINTF(params)    vsnprintf params
#define OSP_SPRINTF_CHECK sprintf

/*****************************************************************************/
/* Macros to get/release a mutex                                             */
/*****************************************************************************/
#define OSP_GET_LOCK(X)                                          \
  {                                                              \
    int osp_lock_error;                                          \
    TRC_DETAIL((TB, "Locking " #X));                             \
    osp_lock_error = pthread_mutex_lock(X);                      \
    TRC_DETAIL((TB, "Locked (( " #X));                           \
    TRC_ASSERT((osp_lock_error == 0),                            \
                     (TB, "Lock failed %d", osp_lock_error));    \
  }

#define OSP_RELEASE_LOCK(X)                                      \
  {                                                              \
    int osp_lock_error;                                          \
    TRC_DETAIL((TB, ")) Unlocking " #X));                        \
    osp_lock_error = pthread_mutex_unlock(X);                    \
    TRC_DETAIL((TB,  "Unlocked " #X));                           \
    TRC_ASSERT((osp_lock_error == 0),                            \
                     (TB, "Unlock failed %d", osp_lock_error));  \
  }

/*****************************************************************************/
/* max/min functions                                                         */
/*****************************************************************************/
#define OSP_MAX(a, b) ((a) > (b) ? (a) : (b))
#define OSP_MIN(a, b) ((a) > (b) ? (b) : (a))

/*****************************************************************************/
/* Unused parameter macro - prevents compiler warnings                       */
/*****************************************************************************/
#define OSP_UNUSED_PARAM(x)  x

/*****************************************************************************/
/* Macro to dump error information on failure of an OpenSSL call             */
/*****************************************************************************/
#define OSP_OPENSSL_ERROR(X)                                    \
{                                                               \
  char osp_dbg_out[256];                                        \
  ERR_error_string(ERR_get_error(), osp_dbg_out);               \
  TRC_ERROR(X);                                                 \
  TRC_ERROR((TB, "OpenSSL: %s", osp_dbg_out));                  \
}

#ifdef OSP_MEASURE_TRANSACTION
/*****************************************************************************/
/* Number of single transaction timings measurements.                        */
/*****************************************************************************/
#define SM_NUM_TIMINGS                  8

/*****************************************************************************/
/* Timings array indices.                                                    */
/*****************************************************************************/
#define SM_TIMINGS_START_READ           0
#define SM_TIMINGS_FINISH_READ          1
#define SM_TIMINGS_START_PARSE          2
#define SM_TIMINGS_FINISH_PARSE         3
#define SM_TIMINGS_START_BUILD          4
#define SM_TIMINGS_FINISH_BUILD         5
#define SM_TIMINGS_START_SEND           6
#define SM_TIMINGS_FINISH_SEND          7
#endif

/*****************************************************************************/
/* Doubly-linked list management                                             */
/*****************************************************************************/

/**STRUCT+********************************************************************/
/* Structure:   LIST_ENTRY                                                   */
/*                                                                           */
/* Description: Contains pointers to next and previous items in a doubly     */
/*              linked list                                                  */
/*****************************************************************************/

typedef struct list_entry
{
  /***************************************************************************/
  /* Pointer to next item in list                                            */
  /***************************************************************************/
  struct list_entry *next;

  /***************************************************************************/
  /* Pointer to previous item in list                                        */
  /***************************************************************************/
  struct list_entry *prev;

} LIST_ENTRY;

/**STRUCT-********************************************************************/

/**STRUCT+********************************************************************/
/* Structure:   OSP_RECURSIVE_MUTEX                                          */
/*                                                                           */
/* Description: Stucture used to implement recursive mutexes for OSP         */
/*                                                                           */
/*****************************************************************************/

typedef struct osp_recursive_mutex
{
  /***************************************************************************/
  /* The thread that currently holds the mutex                               */
  /***************************************************************************/
  pthread_t current_locker;

  /***************************************************************************/
  /* The count of locks on the mutex                                         */
  /***************************************************************************/
  unsigned int lock_count;

  /***************************************************************************/
  /* The underlying mutex                                                    */
  /***************************************************************************/
  pthread_mutex_t mutex;

} OSP_RECURSIVE_MUTEX;

/**STRUCT-********************************************************************/

/*****************************************************************************/
/* Initialize a list                                                         */
/*****************************************************************************/
#define LIST_INIT(list) \
  (list)->prev = (list)->next = (list)

/*****************************************************************************/
/* Insert item at head/position/tail of list                                 */
/*****************************************************************************/
#define LIST_INSERT_HEAD(list, item) \
  {                                  \
    LIST_ENTRY *old_head;            \
    old_head = (list)->next;         \
    (list)->next = (item);           \
    (item)->next = old_head;         \
    (item)->prev = (list);           \
    old_head->prev = (item);         \
  }

#define LIST_INSERT_BEFORE(before_item, new_item) \
  {                                               \
    LIST_ENTRY *old_prev_item;                    \
    old_prev_item = (before_item)->prev;          \
    (new_item)->prev = old_prev_item;             \
    (new_item)->next = (before_item);             \
    old_prev_item->next = (new_item);             \
    (before_item)->prev = (new_item);             \
  }

#define LIST_INSERT_AFTER(after_item, new_item) \
  {                                             \
    LIST_ENTRY *old_next_item;                  \
    old_next_item = (after_item)->next;         \
    (new_item)->next = old_next_item;           \
    (new_item)->prev = (after_item);            \
    old_next_item->prev = (new_item);           \
    (after_item)->next = (new_item);            \
  }

#define LIST_INSERT_TAIL(list, item) \
  {                                  \
    LIST_ENTRY *old_tail;            \
    old_tail = (list)->prev;         \
    old_tail->next = (item);         \
    (item)->next = list;             \
    (list)->prev = (item);           \
    (item)->prev = old_tail;         \
  }

/*****************************************************************************/
/* Get pointer to first item in list or to item after specified item.        */
/*                                                                           */
/* Notes:                                                                    */
/* - does not remove item from list                                          */
/* - returns NULL when it reaches the end of the list                        */
/*****************************************************************************/
#define LIST_GET_FIRST(list) \
  (((list)->next != (list)) ? (list)->next : NULL)

#define LIST_GET_NEXT(list, item) \
  (((item)->next != (list)) ? (item)->next : NULL)

/*****************************************************************************/
/* Remove first/last/specified item from list                                */
/*****************************************************************************/
#define LIST_REMOVE_HEAD(list)                                      \
  (list)->next;                                                     \
  {                                                                 \
    TRC_ASSERT((!LIST_IS_EMPTY(list)),                              \
               (TB, "Tried to remove head of empty list %p", list)) \
    LIST_REMOVE_ITEM((list)->next);                                 \
  }

#define LIST_REMOVE_TAIL(list)                                      \
  (list)->prev;                                                     \
  {                                                                 \
    TRC_ASSERT((!LIST_IS_EMPTY(list)),                              \
               (TB, "Tried to remove tail of empty list %p", list)) \
    LIST_REMOVE_ITEM((list)->prev);                                 \
  }

#define LIST_REMOVE_ITEM(item)          \
{                                       \
  LIST_ENTRY *next_item = (item)->next; \
  LIST_ENTRY *prev_item = (item)->prev; \
  prev_item->next = next_item;          \
  next_item->prev = prev_item;          \
}

/*****************************************************************************/
/* Are there any items in the list?                                          */
/*****************************************************************************/
#define LIST_IS_EMPTY(list) \
  ((list)->next == (list))

/*****************************************************************************/
/* Singly linked list managament                                             */
/*****************************************************************************/

/**STRUCT+********************************************************************/
/* Structure: SLIST_ENTRY                                                    */
/*                                                                           */
/* Description: Queue element.                                               */
/*****************************************************************************/

typedef struct slist_entry
{
  /***************************************************************************/
  /* Pointer to next queue element                                           */
  /***************************************************************************/
  struct slist_entry *next;

} SLIST_ENTRY;

/**STRUCT-********************************************************************/

/**STRUCT+********************************************************************/
/* Structure: SLIST_ROOT                                                     */
/*                                                                           */
/* Description: Head of a singly linked list                                 */
/*                                                                           */
/*****************************************************************************/

typedef struct slist_root
{
  struct slist_entry *first;
  struct slist_entry *last;

} SLIST_ROOT;

/**STRUCT-********************************************************************/

#define SL_INIT_ROOT(R)                                                 \
    (R).first = NULL;                                                   \
    (R).last = NULL

#define SL_ADD_TO_TAIL(LR, LE)                                          \
    ((SLIST_ENTRY *)(LE))->next = NULL;                                 \
    if ((LR).last != NULL)                                              \
    {                                                                   \
      (LR).last->next = (SLIST_ENTRY *)(LE);                            \
    }                                                                   \
    else                                                                \
    {                                                                   \
      (LR).first = (SLIST_ENTRY *)(LE);                                 \
    }                                                                   \
    (LR).last = (SLIST_ENTRY *)(LE)

#define SL_HEAD(LR)    ((void *)(LR).first)

#define SL_REMOVE_HEAD(LR)                                              \
    if ((LR).first != NULL)                                             \
    {                                                                   \
      (LR).first = (LR).first->next;                                    \
    }                                                                   \
    if ((LR).first == NULL)                                             \
    {                                                                   \
      (LR).last = NULL;                                                 \
    }

/**STRUCT+********************************************************************/
/* Structure: WA_LOCKED                                                      */
/*                                                                           */
/* Description: Shared between worker and application threads and protected  */
/*              by mutex_wa                                                  */
/*****************************************************************************/

typedef struct wa_locked
{
  /***************************************************************************/
  /* Protects WA_LOCKED structure                                            */
  /***************************************************************************/
  pthread_mutex_t mutex_wa;

  /***************************************************************************/
  /* Set of file descriptors that worker wants to write on. Saves building   */
  /* the set for each select call.                                           */
  /***************************************************************************/
  fd_set write_set;

  /***************************************************************************/
  /* Indicates whether a wakeup byte has been sent to this worker on         */
  /* wakeup_pipe_in. 0 means no wakeup has been sent, 1 indicates a wakeup   */
  /* has been sent.                                                          */
  /***************************************************************************/
  int sent_wakeup;

  /***************************************************************************/
  /* The time that the wakeup byte was last written.                         */
  /***************************************************************************/
  struct timeval woken;

  /***************************************************************************/
  /* Pipe that a thread writes down to wakeup the worker thread.             */
  /***************************************************************************/
  int wakeup_pipe_in;

  /***************************************************************************/
  /* Per-thread cached time string.                                          */
  /***************************************************************************/
  char cached_time_string[COM_DEF_TIME_LEN];

  /***************************************************************************/
  /* The time when the cached time string was constructed.                   */
  /***************************************************************************/
  time_t cached_time;

} WA_LOCKED;

/**STRUCT-********************************************************************/

/**STRUCT+********************************************************************/
/* Structure: LW_LOCKED                                                      */
/*                                                                           */
/* Description: Structure containing parameters shared between listener and  */
/*              worker threads and locked by mutex_lw                        */
/*****************************************************************************/

typedef struct lw_locked
{
  /***************************************************************************/
  /* Protects assigned_conns                                                 */
  /***************************************************************************/
  pthread_mutex_t mutex_lw;

  /***************************************************************************/
  /* Number of connections assigned to the worker. Incremented by the        */
  /* listener when a new connection is assigned and decremented by the       */
  /* worker when a connection is closed.                                     */
  /***************************************************************************/
  unsigned int assigned_conns;

  /***************************************************************************/
  /* Busy-ness measure for this worker thread.  This is calculated and       */
  /* updated by each worker thread when about to enter a blocking select,    */
  /* and queried by the listener thread when deciding how to allocate a new  */
  /* connection.                                                             */
  /***************************************************************************/
  int busyness;

} LW_LOCKED;

/**STRUCT-********************************************************************/

/**STRUCT+********************************************************************/
/* Structure:   THREAD_INFO                                                  */
/*                                                                           */
/* Description: Structure representing a worker thread.                      */
/*****************************************************************************/

typedef struct thread_info
{
  /***************************************************************************/
  /* Accessed only by the listener thread except on thread termination.      */
  /* links and notification_pipe_in are protected by                         */
  /* listener_info.worker_list_mutex                                         */
  /*                                                                         */
  /* Entry in listener's worker_list                                         */
  /***************************************************************************/
  LIST_ENTRY links;

  /***************************************************************************/
  /* Listener writes into this pipe to notify worker thread of a new         */
  /* connection that it must handle.                                         */
  /***************************************************************************/
  int notification_pipe_in;

  /***************************************************************************/
  /* Not changed after initialization - OS thread handle.                    */
  /***************************************************************************/
  pthread_t thread_handle;

  /***************************************************************************/
  /* Accessed only by worker thread.                                         */
  /*                                                                         */
  /* Worker thread reading from this pipe wakes it up from the select so     */
  /* that it can re-select with different file descriptors (for writing)     */
  /***************************************************************************/
  int wakeup_pipe_out;

  /***************************************************************************/
  /* Worker thread reads the address of a new conn_info structure from this  */
  /* pipe.                                                                   */
  /***************************************************************************/
  int notification_pipe_out;

  /***************************************************************************/
  /* Worker's list of connections.                                           */
  /***************************************************************************/
  LIST_ENTRY conn_list;

  /***************************************************************************/
  /* Set of file descriptors that worker wants to read on. Saves building    */
  /* the set for each select call.                                           */
  /***************************************************************************/
  fd_set read_set;

  /***************************************************************************/
  /* Set of file descriptors that worker wants to be alerted about errors    */
  /* for.  Saves building the set for each select call.                      */
  /***************************************************************************/
  fd_set error_set;

  /***************************************************************************/
  /* Array into which the worker reads CONN_INFO pointers from               */
  /* notification_pipe_out.  Guards against not receiving the whole pointer  */
  /* in one read().                                                          */
  /***************************************************************************/
  char new_connection[sizeof(struct CONN_INFO *)];

  /***************************************************************************/
  /* Amount of the new_connection which is valid.                            */
  /***************************************************************************/
  int new_connection_length;

  /***************************************************************************/
  /* Time that the first renegotiation is due for the connections managed by */
  /* this thread.                                                            */
  /***************************************************************************/
  time_t first_renegotiate;

  /***************************************************************************/
  /* Highest file descriptor that I will select on + 1 - used on select call */
  /***************************************************************************/
  int nfds;

  /***************************************************************************/
  /* The number of connections for which data may be buffered within SSL.    */
  /***************************************************************************/
  int ssl_data_buffered_count;

  /***************************************************************************/
  /* The time when the last blocking select call on this thread was          */
  /* initiated.                                                              */
  /***************************************************************************/
  struct timeval select_start_time;

  /***************************************************************************/
  /* The time when the last blocking select call on this thread returned.    */
  /***************************************************************************/
  struct timeval select_return_time;

  /***************************************************************************/
  /* Shared between listener and worker threads                              */
  /***************************************************************************/
  LW_LOCKED lw_locked;

  /***************************************************************************/
  /* Shared between worker and application threads                           */
  /***************************************************************************/
  WA_LOCKED wa_locked;

  /***************************************************************************/
  /* Pointer to per-thread XML parser object.  Stored here to avoid having   */
  /* to create and destroy an XML_Parser for every transaction.              */
  /***************************************************************************/
  void *parser;

} THREAD_INFO;

/**STRUCT-********************************************************************/

/**STRUCT+********************************************************************/
/* Structure:   TRANS_INFO                                                   */
/*                                                                           */
/* Description: Used to process an entire message                            */
/*****************************************************************************/

typedef struct trans_info
{
  /***************************************************************************/
  /* Structure containing forward and back links                             */
  /***************************************************************************/
  LIST_ENTRY links;

  /***************************************************************************/
  /* The connection on which the transaction was received; not changed after */
  /* initialization                                                          */
  /***************************************************************************/
  struct conn_info *connection;

  /***************************************************************************/
  /* Transaction serial number (per connection).                             */
  /***************************************************************************/
  int serial;

  /***************************************************************************/
  /* Identification information about the client from which the transaction  */
  /* was received; not changed after initialization                          */
  /***************************************************************************/
  OSP_CLIENT_ID client_id;

  /***************************************************************************/
  /* Pointer to xml layer state information                                  */
  /***************************************************************************/
  void *xml_state;

  /***************************************************************************/
  /* Message ID for the transaction                                          */
  /***************************************************************************/
  char *message_id;

  /***************************************************************************/
  /* Variables accessed only by the worker thread                            */
  /***************************************************************************/
  /* Buffer containing the received message                                  */
  /***************************************************************************/
  char *recv_buffer;

  /***************************************************************************/
  /* Total size of the receive buffer                                        */
  /***************************************************************************/
  int recv_buffer_size;

  /***************************************************************************/
  /* Offset of the start of the next data to be processed                    */
  /***************************************************************************/
  int recv_buffer_start;

  /***************************************************************************/
  /* Offset of the end of the current data in the receive buffer             */
  /***************************************************************************/
  int recv_buffer_end;

  /***************************************************************************/
  /* HTTP version in use for this transaction                                */
  /***************************************************************************/
  int http_version;

  /***************************************************************************/
  /* Pointer to AIB-maintained state information                             */
  /***************************************************************************/
  void *aib_state;

  /***************************************************************************/
  /* Variables shared by the application and worker threads                  */
  /***************************************************************************/
  /* Mutex used to protect access to the variables                           */
  /***************************************************************************/
  pthread_mutex_t mutex;

  /***************************************************************************/
  /* Flag used to indicate that the transaction has been submitted to SM for */
  /* sending                                                                 */
  /***************************************************************************/
  int on_send_queue;

  /***************************************************************************/
  /* Buffer containing response                                              */
  /***************************************************************************/
  char *send_buffer;

  /***************************************************************************/
  /* Total size of send buffer                                               */
  /***************************************************************************/
  int send_buffer_size;

  /***************************************************************************/
  /* Offset of start of data in the send buffer.  Allows space to be         */
  /* reserved for lower layers to pre-pend their headers on the data         */
  /***************************************************************************/
  int send_buffer_start;

  /***************************************************************************/
  /* Offset of end of data in the send buffer; new data should be appended   */
  /* after this                                                              */
  /***************************************************************************/
  int send_buffer_end;

  /***************************************************************************/
  /* If true, the information in the send buffer should be signed before     */
  /* sending                                                                 */
  /***************************************************************************/
  int sign_response;

  /***************************************************************************/
  /* The digest that S/MIME should use when signing the response.            */
  /***************************************************************************/
  unsigned int signing_digest;

  /***************************************************************************/
  /* Set if a component of the message is being parsed                       */
  /***************************************************************************/
  int parsing;

  /***************************************************************************/
  /* Count of requests in the received message                               */
  /***************************************************************************/
  int request_count;

  /***************************************************************************/
  /* Count of responses received                                             */
  /***************************************************************************/
  int response_count;

  /***************************************************************************/
  /* HTTP status code associated with the transaction.  If not 200, then a   */
  /* simple HTTP error is sent                                               */
  /***************************************************************************/
  int http_status;

  /***************************************************************************/
  /* Time when we first starting reading data for this transaction.          */
  /***************************************************************************/
  struct timeval start_time_io;

  /***************************************************************************/
  /* Time when we began processing data for this transaction.                */
  /***************************************************************************/
  struct timeval start_time_processing;

#ifdef OSP_MEASURE_TRANSACTION
  /***************************************************************************/
  /* Detailed timings.                                                       */
  /***************************************************************************/
  SM_TIMINGS timings[SM_NUM_TIMINGS];
#endif

  /***************************************************************************/
  /* Pool of parsed character data strings.                                  */
  /***************************************************************************/
  char *pool;

  /***************************************************************************/
  /* String pool size.                                                       */
  /***************************************************************************/
  int pool_size;

  /***************************************************************************/
  /* Current position in the string pool.                                    */
  /***************************************************************************/
  int pool_position;

  /***************************************************************************/
  /* The thread that this transaction is parsed on.                          */
  /***************************************************************************/
  THREAD_INFO *thread;

} TRANS_INFO;

/**STRUCT-********************************************************************/

/**STRUCT+********************************************************************/
/* Structure: OSP_INT_CORRELATOR                                             */
/*                                                                           */
/* Description: Structure pointed to by the OSP_CORRELATOR                   */
/*                                                                           */
/*****************************************************************************/

typedef struct osp_int_correlator
{
  /***************************************************************************/
  /* Pointer to the transation info                                          */
  /***************************************************************************/
  TRANS_INFO *trans;

  /***************************************************************************/
  /* Component ID to return on respsonse/confirmation                        */
  /***************************************************************************/
  char *comp_id;

  /***************************************************************************/
  /* Type of the component (used to determine the correct spelling for       */
  /* certain responses)                                                      */
  /***************************************************************************/
  int comp_type;

  /***************************************************************************/
  /* Whether the memory for this correlator was allocated dynamically.       */
  /***************************************************************************/
  int allocated;

} OSP_INT_CORRELATOR;

/**STRUCT-********************************************************************/

/**STRUCT+********************************************************************/
/* Structure: COM_STRING                                                     */
/*                                                                           */
/* Description: Describes a non-null-terminated string.                      */
/*                                                                           */
/*****************************************************************************/

typedef struct com_string
{
  /***************************************************************************/
  /* Pointer to the string.                                                  */
  /***************************************************************************/
  char *buffer;

  /***************************************************************************/
  /* Length of the string.                                                   */
  /***************************************************************************/
  unsigned int len;

} COM_STRING;

/**STRUCT-********************************************************************/

/**STRUCT+********************************************************************/
/* Structure: COM_PARAM                                                      */
/*                                                                           */
/* Description: Describes a single parameter from an HTTP/MIME header.       */
/*                                                                           */
/*****************************************************************************/

typedef struct com_param
{
  /***************************************************************************/
  /* The parameter's attribute name.                                         */
  /***************************************************************************/
  COM_STRING attribute;

  /***************************************************************************/
  /* The parameter's attribute value.                                        */
  /***************************************************************************/
  COM_STRING value;

} COM_PARAM;

/**STRUCT-********************************************************************/

/**STRUCT+********************************************************************/
/* Structure: COM_PARAMS                                                     */
/*                                                                           */
/* Description: Describes a list of parameters from an HTTP/MIME header.     */
/*                                                                           */
/*****************************************************************************/

typedef struct com_params
{
  /***************************************************************************/
  /* The number of parameters in the array.                                  */
  /***************************************************************************/
  unsigned int num_params;

  /***************************************************************************/
  /* The list of parameters.                                                 */
  /***************************************************************************/
  COM_PARAM *params;

} COM_PARAMS;

/**STRUCT-********************************************************************/

/**STRUCT+********************************************************************/
/* Structure: COM_HEADER                                                     */
/*                                                                           */
/* Description: Describes a single HTTP/MIME header.                         */
/*                                                                           */
/*****************************************************************************/

typedef struct com_header
{
  /***************************************************************************/
  /* The header name.                                                        */
  /***************************************************************************/
  COM_STRING name;

  /***************************************************************************/
  /* The header value (unparsed -- may include parameters).                  */
  /***************************************************************************/
  COM_STRING value;

} COM_HEADER;

/**STRUCT-********************************************************************/

/**STRUCT+********************************************************************/
/* Structure: COM_HEADERS                                                    */
/*                                                                           */
/* Description: Describes a list of HTTP/MIME headers.                       */
/*                                                                           */
/*****************************************************************************/

typedef struct com_headers
{
  /***************************************************************************/
  /* The number of headers in the array.                                     */
  /***************************************************************************/
  unsigned int num_headers;

  /***************************************************************************/
  /* The list of headers.                                                    */
  /***************************************************************************/
  COM_HEADER *headers;

} COM_HEADERS;

/**STRUCT-********************************************************************/

/*****************************************************************************/
/* commain.c                                                                 */
/*****************************************************************************/
OSP_RC osp_common_init(void);

OSP_RC osp_common_term(void);

OSP_RC osp_create_mutex(pthread_mutex_t *);

OSP_RC osp_destroy_mutex(pthread_mutex_t *);

OSP_RC osp_create_recursive_mutex(OSP_RECURSIVE_MUTEX *);

void osp_get_recursive_lock(OSP_RECURSIVE_MUTEX *);

void osp_release_recursive_lock(OSP_RECURSIVE_MUTEX *);

OSP_RC osp_destroy_recursive_mutex(OSP_RECURSIVE_MUTEX *);

char *osp_common_get_config_filename(void);

OSP_RC osp_base64_encode(char *, unsigned int, char **, unsigned int *);

OSP_RC osp_base64_decode(char *, unsigned int, char **, unsigned int *);

/*****************************************************************************/
/* comparse.c                                                                */
/*****************************************************************************/
OSP_RC com_parse_headers(char *,
                         unsigned int,
                         COM_HEADERS **,
                         char **);

OSP_RC com_find_header_value(const COM_HEADERS *,
                             const char *,
                             const unsigned int,
                             COM_STRING *);

OSP_RC com_parse_media_type(char *,
                            unsigned int,
                            COM_STRING *,
                            COM_PARAMS **);

OSP_RC com_find_param_value(const COM_PARAMS *,
                            const char *,
                            const unsigned int,
                            COM_STRING *);

int com_strings_match(const COM_STRING *,
                      const char *,
                      const unsigned int);

#endif

