#ifndef OPENOSP_INCLUDED
#define OPENOSP_INCLUDED

/**INC+***********************************************************************/
/* Header:    openosp.h                                                      */
/*                                                                           */
/* Purpose:   OpenOSP API definitions.                                       */
/*                                                                           */
/* Copyright (c) 2000, 2001 Data Connection Limited                          */
/* Copyright (c) 2000, 2001 Cisco Systems, Inc. All Rights Reserved.         */
/*                                                                           */
/* All software incorporated in the "OpenOSP Server" (except for those       */
/* portions incorporating third party software specifically identified as    */
/* being licensed under separate license) are licensed under the terms of    */
/* the Cisco Public Source License ver. 1.0 (2000) found in                  */
/* $OSPROOT/license.txt.  Additional copies of the license are available     */
/* from opensource@cisco.com.                                                */
/*                                                                           */
/* Also, before using this software, please see $OSPROOT/x509_ipr.txt for    */
/* information regarding intellectual property rights.                       */
/*                                                                           */
/* $Revision::   1.31               $ $Modtime::   Jan 11 2001 15:56:38   $  */
/*                                                                           */
/**INC-***********************************************************************/

#ifdef __cplusplus
extern "C" {
#endif

/*****************************************************************************/
/*                                                                           */
/* Conventions                                                               */
/*                                                                           */
/* Where a parameter or member of a structure is a pointer to some data      */
/* which may not always be provided, such as the service type in the         */
/* OSP_USAGE_DETAIL structure, then this API adopts the convention of        */
/* passing a NULL pointer rather than a pointer to an empty string.          */
/*                                                                           */
/*****************************************************************************/

/*****************************************************************************/
/*                                                                           */
/* OSP Types                                                                 */
/*                                                                           */
/*****************************************************************************/

/*****************************************************************************/
/* OSP calling convention                                                    */
/*****************************************************************************/
#define OSPAPI

/*****************************************************************************/
/* Result code                                                               */
/*****************************************************************************/
typedef unsigned int            OSP_RC;
#define OSP_SUCCESS                 0
#define OSP_ERROR                   1
#define OSP_ERR_ALREADY_INIT        2
#define OSP_ERR_BAD_HANDLE          3
#define OSP_ERR_BAD_PARAM           4
#define OSP_ERR_BAD_REQUEST         5
#define OSP_ERR_FAILED_SYNC_SEND    6
#define OSP_ERR_NOT_INIT            7
#define OSP_ERR_UNSUPPORTED         8
#define OSP_ERR_MEMORY              9
#define OSP_ERR_DENIED             10
#define OSP_ERR_BAD_CORR           11
#define OSP_ERR_NO_RESOURCES       12
#define OSP_ERR_FILE               13
#define OSP_ERR_CONFIG             14
#define OSP_ERR_MISSING_ATTR       15
#define OSP_ERR_FAILED_VERIFY      16
#define OSP_ERR_BUFSIZE            17
#define OSP_ERR_BIND_FAILED        18

/*****************************************************************************/
/* OSP status codes                                                          */
/*****************************************************************************/
#define OSP_STATUS_SUCCESS                                 200
#define OSP_STATUS_INFORMATION_CREATED                     201
#define OSP_STATUS_UPDATED_INFORMATION_ACCEPTED            210
#define OSP_STATUS_BAD_REQUEST                             400
#define OSP_STATUS_UNAUTHORIZED                            401
#define OSP_STATUS_AUTHENTICATION_UNSUCCESSFUL             402
#define OSP_STATUS_CALL_AUTHORIZATION_UNSUCCESSFUL         403
#define OSP_STATUS_ROUTE_AUTHORIZATION_UNSUCCESSFUL        404
#define OSP_STATUS_CHARACTER_ENCODING_NOT_SUPPORTED        410
#define OSP_STATUS_PARSING_UNSUCCESSFUL                    411
#define OSP_STATUS_CRITICAL_ELEMENT_NOT_SUPPORTED          412
#define OSP_STATUS_GENERIC_SECURITY_PROBLEM                420
#define OSP_STATUS_SIGNATURE_INVALID                       421
#define OSP_STATUS_CRYPTOGRAPHIC_ALGORITHM_NOT_SUPPORTED   422
#define OSP_STATUS_CERTIFICATE_INVALID                     423
#define OSP_STATUS_CERTIFICATE_REVOKED                     424
#define OSP_STATUS_ENCRYPTION_REQUIRED                     425
#define OSP_STATUS_INTERNAL_SERVER_ERROR                   500
#define OSP_STATUS_NOT_IMPLEMENTED                         501
#define OSP_STATUS_SERVICE_NOT_AVAILABLE                   503
#define OSP_STATUS_TRANSIENT_PROBLEM_IN_SERVER             510
#define OSP_STATUS_LONG_TERM_PROBLEM_IN_SERVER             520
#define OSP_STATUS_TIME_PROBLEM                            530
#define OSP_STATUS_VALID_TIME_TOO_SOON                     531
#define OSP_STATUS_TIME_INTERVAL_TOO_SMALL                 532
#define OSP_STATUS_GENERIC_FAILURE                         999

/*****************************************************************************/
/* AuditSignal extension values                                              */
/*****************************************************************************/
#define OSP_UM_AUDIT_START                                 1001
#define OSP_UM_AUDIT_STOP                                  1002

/*****************************************************************************/
/* OSP_CORRELATOR                                                            */
/*                                                                           */
/* App callback correlator.  This is an opaque "cookie" to the application   */
/*****************************************************************************/
typedef void * OSP_CORRELATOR;

/**STRUCT+********************************************************************/
/* Structure:   OSP_TOKEN                                                    */
/*                                                                           */
/* Description: Base64 or CDATA encoded token data                           */
/*****************************************************************************/

typedef struct osp_token
{
  /***************************************************************************/
  /* Pointer to the next item in the list                                    */
  /***************************************************************************/
  struct osp_token *next;

  /***************************************************************************/
  /* Encoding used for the token                                             */
  /***************************************************************************/
  char *encoding;     /* can be "base64" or "cdata"                          */

  /***************************************************************************/
  /* Length of token data.  Only used for sending tokens; received tokens    */
  /* will be NULL terminated.                                                */
  /***************************************************************************/
  int token_len;

  /***************************************************************************/
  /* The encoded token                                                       */
  /***************************************************************************/
  char *token_data;

} OSP_TOKEN;

/**STRUCT-********************************************************************/

/**STRUCT+********************************************************************/
/* Structure:   OSP_AUTH_INFO                                                */
/*                                                                           */
/* Description: Base64 or CDATA encoded subscriber authentication information*/
/*****************************************************************************/

typedef struct osp_auth_info
{
  /***************************************************************************/
  /* Pointer to the next item in the list                                    */
  /***************************************************************************/
  struct osp_auth_info *next;

  /***************************************************************************/
  /* Encoding used for the information                                       */
  /***************************************************************************/
  char *encoding;     /* can be "base64" or "cdata"                          */

  /***************************************************************************/
  /* The encoded authentication information                                  */
  /***************************************************************************/
  char *auth_data;

} OSP_AUTH_INFO;

/**STRUCT-********************************************************************/

/**STRUCT+********************************************************************/
/* Structure:   OSP_CERTIFICATE                                              */
/*                                                                           */
/* Description: Base64 or CDATA encoded public key certificate               */
/*****************************************************************************/

typedef struct osp_certificate
{
  /***************************************************************************/
  /* Pointer to the next item in the list                                    */
  /***************************************************************************/
  struct osp_certificate *next;

  /***************************************************************************/
  /* Encoding used for the certificate                                       */
  /***************************************************************************/
  char *encoding;     /* can be "base64" or "cdata"                          */

  /***************************************************************************/
  /* Length of certificate data                                              */
  /***************************************************************************/
  int cert_len;

  /***************************************************************************/
  /* The encoded certificate                                                 */
  /***************************************************************************/
  char *cert_data;

} OSP_CERTIFICATE;

/**STRUCT-********************************************************************/

/**STRUCT+********************************************************************/
/* Structure:   OSP_ADDRESS                                                  */
/*                                                                           */
/* Description: Source/Destination addressing information                    */
/*****************************************************************************/

typedef struct osp_address
{
  /***************************************************************************/
  /* Pointer to the next item in the list                                    */
  /***************************************************************************/
  struct osp_address *next;

  /***************************************************************************/
  /* Type of information, e.g.  e164 number or serial number                 */
  /***************************************************************************/
  char *type;

  /***************************************************************************/
  /* The address data                                                        */
  /***************************************************************************/
  char *address;

} OSP_ADDRESS;

/**STRUCT-********************************************************************/

/**STRUCT+********************************************************************/
/* Structure:   OSP_USAGE_STATISTICS_PF                                      */
/*                                                                           */
/* Description: Substructure of OSP enhanced usage statistics structure,     */
/*              with fields for a total packet count and the fraction lost.  */
/*****************************************************************************/

typedef struct osp_usage_statistics_pf
{
  /***************************************************************************/
  /* Pointer to the next item in the list                                    */
  /***************************************************************************/
  struct osp_usage_statistics_pf *next;

  /***************************************************************************/
  /* Count of the total number of packets.                                   */
  /***************************************************************************/
  char *packets;

  /***************************************************************************/
  /* Fraction of packets lost, in scale 0-255.                               */
  /***************************************************************************/
  char *fraction;

} OSP_USAGE_STATISTICS_PF;

/**STRUCT-********************************************************************/

/**STRUCT+********************************************************************/
/* Structure:   OSP_USAGE_STATISTICS_MMVS                                    */
/*                                                                           */
/* Description: Substructure of OSP enhanced usage statistics structure,     */
/*              with fields for minimum, mean, variance and sample count.    */
/*****************************************************************************/

typedef struct osp_usage_statistics_mmvs
{
  /***************************************************************************/
  /* Pointer to the next item in the list                                    */
  /***************************************************************************/
  struct osp_usage_statistics_mmvs *next;

  /***************************************************************************/
  /* Minimum sample value.                                                   */
  /***************************************************************************/
  char *minimum;

  /***************************************************************************/
  /* Mean sample value.                                                      */
  /***************************************************************************/
  char *mean;

  /***************************************************************************/
  /* Variance of sample values.                                              */
  /***************************************************************************/
  char *variance;

  /***************************************************************************/
  /* Number of samples.                                                      */
  /***************************************************************************/
  char *samples;

} OSP_USAGE_STATISTICS_MMVS;

/**STRUCT-********************************************************************/

/**STRUCT+********************************************************************/
/* Structure:   OSP_USAGE_STATISTICS                                         */
/*                                                                           */
/* Description: Enhanced OSP usage statistics structure.                     */
/*****************************************************************************/

typedef struct osp_usage_statistics
{
  /***************************************************************************/
  /* Pointer to the next item in the list                                    */
  /***************************************************************************/
  struct osp_usage_statistics *next;

  /***************************************************************************/
  /* Packet loss information for datagrams transmitted by the reporting      */
  /* system that were not received by its peer, as reported in the peer's    */
  /* RTCP sender and receiver reports.                                       */
  /***************************************************************************/
  OSP_USAGE_STATISTICS_PF *loss_sent;

  /***************************************************************************/
  /* Packet loss information for datagrams that should have been received by */
  /* the reporting system but were not, as reported in the the system's RTCP */
  /* sender and receiver reports.                                            */
  /***************************************************************************/
  OSP_USAGE_STATISTICS_PF *loss_received;

  /***************************************************************************/
  /* Measurements of one way delay to the reporting system from its peer, as */
  /* measured during the communication.                                      */
  /***************************************************************************/
  OSP_USAGE_STATISTICS_MMVS *one_way_delay;

  /***************************************************************************/
  /* Measurements of round trip delay between the reporting system and its   */
  /* peer, as measured during the communication.                             */
  /***************************************************************************/
  OSP_USAGE_STATISTICS_MMVS *round_trip_delay;

} OSP_USAGE_STATISTICS;

/**STRUCT-********************************************************************/

/**STRUCT+********************************************************************/
/* Structure:   OSP_PROTOCOL_TYPE                                            */
/*                                                                           */
/* Description: List of protocols supported/requested                        */
/*****************************************************************************/

typedef struct osp_protocol_type
{
  /***************************************************************************/
  /* Pointer to the next item in the list                                    */
  /***************************************************************************/
  struct osp_protocol_type *next;

  /***************************************************************************/
  /* Protocol name - ss7, h323 etc                                           */
  /***************************************************************************/
  char *prot_name;

} OSP_PROTOCOL_TYPE;

/**STRUCT-********************************************************************/

/**STRUCT+********************************************************************/
/* Structure: OSP_SERVICE_INFO                                               */
/*                                                                           */
/* Description: OSP service information                                      */
/*                                                                           */
/*****************************************************************************/

typedef struct osp_service_info
{
  /***************************************************************************/
  /* Total bandwidth in bits/sec                                             */
  /***************************************************************************/
  char *bandwidth;

  /***************************************************************************/
  /* Type of service - telephony, fax etc                                    */
  /***************************************************************************/
  char *service_type;

  /***************************************************************************/
  /* Protocols supported - h323 fast start, SS7 etc                          */
  /***************************************************************************/
  OSP_PROTOCOL_TYPE *prot_type;

} OSP_SERVICE_INFO;

/**STRUCT-********************************************************************/

/**STRUCT+********************************************************************/
/* Structure: OSP_TERM_CAUSE                                                 */
/*                                                                           */
/* Description: Termination cause                                            */
/*                                                                           */
/*****************************************************************************/

typedef struct osp_term_cause
{
  /***************************************************************************/
  /* numerical termination code; NULL pointer if not available               */
  /***************************************************************************/
  char *term_code;

  /***************************************************************************/
  /* Description of termination reason; NULL pointer if not available        */
  /***************************************************************************/
  char *term_desc;

} OSP_TERM_CAUSE;

/**STRUCT-********************************************************************/

/**STRUCT+********************************************************************/
/* Structure:   OSP_USAGE_DETAIL                                             */
/*                                                                           */
/* Description: Usage details                                                */
/*****************************************************************************/

typedef struct osp_usage_detail
{
  /***************************************************************************/
  /* Pointer to the next item in the list                                    */
  /***************************************************************************/
  struct osp_usage_detail *next;

  /***************************************************************************/
  /* Service information                                                     */
  /***************************************************************************/
  OSP_SERVICE_INFO *service;

  /***************************************************************************/
  /* Amount of service, e.g.  5.5 increments                                 */
  /***************************************************************************/
  char *amount;

  /***************************************************************************/
  /* size of increment, e.g.  60                                             */
  /***************************************************************************/
  char *increment;

  /***************************************************************************/
  /* unit of increment, e.g.  's'                                            */
  /***************************************************************************/
  char *unit;

  /***************************************************************************/
  /* uses the form YYYY-MM-DDThh:mm:ssZ; a NULL pointer when part of an      */
  /* authorization means the authorization is effective immediately          */
  /***************************************************************************/
  char *start_time;

  /***************************************************************************/
  /* uses the form YYYY-MM-DDThh:mm:ssZ; a NULL pointer when part of an      */
  /* authorization means the authorization is of indefinite duration         */
  /***************************************************************************/
  char *end_time;

  /***************************************************************************/
  /* Termination cause; NULL pointer if not available                        */
  /***************************************************************************/
  OSP_TERM_CAUSE *term_cause;

  /***************************************************************************/
  /* enhanced usage statistics (see Annex C of OSP specification)            */
  /***************************************************************************/
  OSP_USAGE_STATISTICS *statistics;

} OSP_USAGE_DETAIL;

/**STRUCT-********************************************************************/

/**STRUCT+********************************************************************/
/* Structure:   OSP_DATA_RATE                                                */
/*                                                                           */
/* Description: Data rate of a client resource                               */
/*****************************************************************************/

typedef struct osp_data_rate
{
  /***************************************************************************/
  /* Pointer to the next item in the list                                    */
  /***************************************************************************/
  struct osp_data_rate *next;

  /***************************************************************************/
  /* Number of channels                                                      */
  /***************************************************************************/
  char *num_channels;

  /***************************************************************************/
  /* bandwidth of each channel or total bandwidth if no channels specified   */
  /***************************************************************************/
  char *bandwidth;

} OSP_DATA_RATE;

/**STRUCT-********************************************************************/

/**STRUCT+********************************************************************/
/* Structure:   OSP_SUPP_PROTOCOL                                            */
/*                                                                           */
/* Description: Supported protocol information                               */
/*                                                                           */
/*****************************************************************************/

typedef struct osp_supp_protocol
{
  /***************************************************************************/
  /* Pointer to the next item in the list                                    */
  /***************************************************************************/
  struct osp_supp_protocol *next;

  /***************************************************************************/
  /* Protocol type                                                           */
  /***************************************************************************/
  OSP_PROTOCOL_TYPE *prot_type;

  /***************************************************************************/
  /* Data rates supported                                                    */
  /***************************************************************************/
  OSP_DATA_RATE *data_rate_list;

} OSP_SUPP_PROTOCOL;

/**STRUCT-********************************************************************/

/**STRUCT+********************************************************************/
/* Structure:   OSP_RESOURCE                                                 */
/*                                                                           */
/* Description: Details of resources at a client                             */
/*****************************************************************************/

typedef struct osp_resource
{
  /***************************************************************************/
  /* Pointer to the next item in the list                                    */
  /***************************************************************************/
  struct osp_resource *next;

  /***************************************************************************/
  /* Data rate for this resource                                             */
  /***************************************************************************/
  OSP_DATA_RATE *data_rate;

  /***************************************************************************/
  /* Almost out of resources flag                                            */
  /***************************************************************************/
  char *almost_out;

  /***************************************************************************/
  /* Supported protocols                                                     */
  /***************************************************************************/
  OSP_SUPP_PROTOCOL *supp_protocol_list;

} OSP_RESOURCE;

/**STRUCT-********************************************************************/

/**STRUCT+********************************************************************/
/* Structure:   OSP_CAPS                                                     */
/*                                                                           */
/* Description: List of the capabilites of at a client                       */
/*****************************************************************************/

typedef struct osp_caps
{
  /***************************************************************************/
  /* Pointer to the next item in the list                                    */
  /***************************************************************************/
  struct osp_caps *next;

  /***************************************************************************/
  /* Capability name                                                         */
  /***************************************************************************/
  char *cap_name;

} OSP_CAPS;

/**STRUCT-********************************************************************/

/**STRUCT+********************************************************************/
/* Structure:   OSP_DEVICE_INFO                                              */
/*                                                                           */
/* Description: Information supplied about a client device                   */
/*****************************************************************************/

typedef struct osp_device_info
{
  /***************************************************************************/
  /* Pointer to the next item in the list                                    */
  /***************************************************************************/
  struct osp_device_info *next;

  /***************************************************************************/
  /* Type of information, e.g.  e164 number or serial number                 */
  /***************************************************************************/
  char *type;

  /***************************************************************************/
  /* The device information                                                  */
  /***************************************************************************/
  char *data;

} OSP_DEVICE_INFO;

/**STRUCT-********************************************************************/

/**STRUCT+********************************************************************/
/* Structure:   OSP_SERVICE_URL                                              */
/*                                                                           */
/* Description: Server service information                                   */
/*****************************************************************************/

typedef struct osp_service_url
{
  /***************************************************************************/
  /* Pointer to the next item in the list                                    */
  /***************************************************************************/
  struct osp_service_url *next;

  /***************************************************************************/
  /* The URL                                                                 */
  /***************************************************************************/
  char *url;

} OSP_SERVICE_URL;

/**STRUCT-********************************************************************/

/**STRUCT+********************************************************************/
/* Structure:   OSP_SERVICE                                                  */
/*                                                                           */
/* Description: Server service information                                   */
/*****************************************************************************/

typedef struct osp_service
{
  /***************************************************************************/
  /* Pointer to the next item in the list                                    */
  /***************************************************************************/
  struct osp_service *next;

  /***************************************************************************/
  /* OSP service supported by server                                         */
  /***************************************************************************/
  char *capability;

  /***************************************************************************/
  /* list of URLs for the service                                            */
  /***************************************************************************/
  OSP_SERVICE_URL *url_list;

  /***************************************************************************/
  /* set to 1 if signatures are required for this service, 0 otherwise.      */
  /***************************************************************************/
  unsigned int sig_required;

} OSP_SERVICE;

/**STRUCT-********************************************************************/

/**STRUCT+********************************************************************/
/* Structure:   OSP_CLIENT_ID                                                */
/*                                                                           */
/* Description: Client Identifier                                            */
/*****************************************************************************/

typedef struct osp_client_id
{
  /***************************************************************************/
  /* client ip address                                                       */
  /***************************************************************************/
  unsigned long ip_address;

  /***************************************************************************/
  /* cookie returned by Client Verification callback                         */
  /***************************************************************************/
  void *ver_cookie;

} OSP_CLIENT_ID;

/**STRUCT-********************************************************************/

/**STRUCT+********************************************************************/
/* Structure:   OSP_CALL_ID                                                  */
/*                                                                           */
/* Description: H.323 Call Identifier                                        */
/*****************************************************************************/

typedef struct osp_call_id
{
  /***************************************************************************/
  /* Pointer to the next item in the list                                    */
  /***************************************************************************/
  struct osp_call_id *next;

  /***************************************************************************/
  /* Encoding used for the identifier                                        */
  /***************************************************************************/
  char *encoding;     /* can be "base64" or "cdata"                          */

  /***************************************************************************/
  /* The encoded identifier                                                  */
  /***************************************************************************/
  char *data;

} OSP_CALL_ID;

/**STRUCT-********************************************************************/

/**STRUCT+********************************************************************/
/* Structure:   OSP_DESTINATION                                              */
/*                                                                           */
/* Description: Destination information                                      */
/*****************************************************************************/

typedef struct osp_destination
{
  /***************************************************************************/
  /* Pointer to the next item in the list                                    */
  /***************************************************************************/
  struct osp_destination *next;

  /***************************************************************************/
  /* destination address or NULL pointer if none supplied                    */
  /***************************************************************************/
  OSP_ADDRESS *info;

  /***************************************************************************/
  /* list of alternate dest addresses                                        */
  /***************************************************************************/
  OSP_ADDRESS *alt_list;

  /***************************************************************************/
  /* call signalling address of dest                                         */
  /***************************************************************************/
  char *signal_address;

  /***************************************************************************/
  /* list of authorization tokens                                            */
  /***************************************************************************/
  OSP_TOKEN *token_list;

  /***************************************************************************/
  /* time authorization begins, using format YYYY-MM-DDThh:mm:ssZ.  A NULL   */
  /* pointer means effective immediately                                     */
  /***************************************************************************/
  char *valid_after;

  /***************************************************************************/
  /* time authorization ends, using format YYYY-MM-DDThh:mm:ssZ.  A NULL     */
  /* pointer means indefinitely                                              */
  /***************************************************************************/
  char *valid_until;

  /***************************************************************************/
  /* List of usage details                                                   */
  /***************************************************************************/
  OSP_USAGE_DETAIL *usage_list;

  /***************************************************************************/
  /* list of URLs where authorization may be verified or refreshed           */
  /***************************************************************************/
  OSP_SERVICE_URL *auth_url_list;

  /***************************************************************************/
  /* H.323 call identifier                                                   */
  /***************************************************************************/
  OSP_CALL_ID *call_id;

} OSP_DESTINATION;

/**STRUCT-********************************************************************/

/**STRUCT+********************************************************************/
/* Structure:   OSP_STATS                                                    */
/*                                                                           */
/* Description: Server statistics                                            */
/*****************************************************************************/

typedef struct osp_stats
{

  /***************************************************************************/
  /* secure connections received                                             */
  /***************************************************************************/
  unsigned int ssl_conns;

  /***************************************************************************/
  /* non-secure connections received                                         */
  /***************************************************************************/
  unsigned int nonssl_conns;

  /***************************************************************************/
  /* total OSP requests received                                             */
  /***************************************************************************/
  unsigned int osp_requests;

  /***************************************************************************/
  /* total SCEP requests received                                            */
  /***************************************************************************/
  unsigned int scep_requests;

} OSP_STATS;

/**STRUCT-********************************************************************/

/**STRUCT+********************************************************************/
/* Structure: OSP_STATUS                                                     */
/*                                                                           */
/* Description: OSP Status information                                       */
/*                                                                           */
/*****************************************************************************/

typedef struct osp_status
{
  /***************************************************************************/
  /* Numerical status code                                                   */
  /***************************************************************************/
  int code;

  /***************************************************************************/
  /* Descriptive text for the status                                         */
  /***************************************************************************/
  const char *desc;

} OSP_STATUS;

/**STRUCT-********************************************************************/

/**STRUCT+********************************************************************/
/* Structure: OSP_CREDIT_AMOUNT                                              */
/*                                                                           */
/* Description: Amount of credit available to a subscriber                   */
/*                                                                           */
/*****************************************************************************/

typedef struct osp_credit_amount
{
  /***************************************************************************/
  /* Currency                                                                */
  /***************************************************************************/
  char *currency;

  /***************************************************************************/
  /* Amount of credit                                                        */
  /***************************************************************************/
  char *amount;

} OSP_CREDIT_AMOUNT;

/**STRUCT-********************************************************************/

/**STRUCT+********************************************************************/
/* Structure: OSP_CREDIT_TIME                                                */
/*                                                                           */
/* Description: Duration of call a subscriber may make                       */
/*                                                                           */
/* Usage: amount = 5, increment = 60, unit = s means a duration of 5 minutes */
/*                                                                           */
/*****************************************************************************/

typedef struct osp_credit_time
{
  /***************************************************************************/
  /* Amount of time increments                                               */
  /***************************************************************************/
  char *amount;

  /***************************************************************************/
  /* Increment                                                               */
  /***************************************************************************/
  char *increment;

  /***************************************************************************/
  /* Unit of time                                                            */
  /***************************************************************************/
  char *unit;

} OSP_CREDIT_TIME;

/**STRUCT-********************************************************************/

/*****************************************************************************/
/*                                                                           */
/* Control API                                                               */
/*                                                                           */
/* The Control API allows the main server application to                     */
/* - start and stop the OSP stack                                            */
/* - register client verification and non-repudiation callbacks              */
/* - obtain performance data about the stack.                                */
/*                                                                           */
/*****************************************************************************/

/*****************************************************************************/
/* Name:    osp_init                                                         */
/*                                                                           */
/* Purpose: Called to initialize the OSP server stack and its associated     */
/*          libraries.                                                       */
/*                                                                           */
/* Returns: OSP_SUCCESS - completed successfully                             */
/*          OSP_ERROR   - inititalization failed                             */
/*                                                                           */
/* Params:  None.                                                            */
/*                                                                           */
/*****************************************************************************/
OSP_RC OSPAPI osp_init(void);

/*****************************************************************************/
/* Name:    osp_listen                                                       */
/*                                                                           */
/* Purpose: Causes the OSP server stack to begin listening for connections   */
/*          from clients                                                     */
/*                                                                           */
/* Returns: OSP_SUCCESS - completed successfully                             */
/*          OSP_ERROR   - stack was not initialized.                         */
/*                                                                           */
/* Params:  None.                                                            */
/*                                                                           */
/*****************************************************************************/
OSP_RC OSPAPI osp_listen(void);

/*****************************************************************************/
/* Name:    osp_term                                                         */
/*                                                                           */
/* Purpose: Called to terminate the OSP server stack and its associated      */
/*          libraries.                                                       */
/*                                                                           */
/* Returns: OSP_SUCCESS - completed successfully                             */
/*          OSP_ERROR   - stack was not initialized.                         */
/*                                                                           */
/* Params:  None.                                                            */
/*                                                                           */
/*****************************************************************************/
OSP_RC OSPAPI osp_term(void);

/*****************************************************************************/
/* Name:    osp_get_stack_statistics                                         */
/*                                                                           */
/* Purpose: Called by the application to get stack statistics                */
/*                                                                           */
/* Returns: OSP_SUCCESS     - completed successfully                         */
/*          OSP_ERROR       - operation failed                               */
/*                                                                           */
/* Params:  stats - pointer to the statistics structure to fill in           */
/*                                                                           */
/*****************************************************************************/
OSP_RC OSPAPI osp_get_stack_statistics(OSP_STATS *stats);

/*****************************************************************************/
/*                                                                           */
/* Usage Metering API                                                        */
/*                                                                           */
/* The Usage Metering (UM) API passes usage information to a permanent       */
/* storage facility or directly to an external usage metering application.   */
/*                                                                           */
/* The API is asynchronous; that is, the application may respond to a usage  */
/* metering callback after the initial callback has returned, and it may do  */
/* so on a different thread.                                                 */
/*****************************************************************************/

/*****************************************************************************/
/* Name:    osp_um_callback                                                  */
/*                                                                           */
/* Purpose: Called by the OSP Server to pass UM information received from    */
/*          an OSP Client.                                                   */
/*                                                                           */
/* Returns: OSP_SUCCESS - completed successfully                             */
/*          other       - OSP result code                                    */
/*                                                                           */
/* Params:  corr            - correlator to be used on subsequent response   */
/*          timestamp       - message timestamp                              */
/*          role            - client role in call                            */
/*          transaction_id  - transaction identifier                         */
/*          call_id         - structure containing H.323 CallId              */
/*          source_info     - pointer to source info structure               */
/*          source_alt_list - list of source alternates                      */
/*          dest_info       - pointer to dest info structure                 */
/*          dest_alt_list   - list of destination alternates                 */
/*          use_detail_list - list of usage details                          */
/*          client_id       - may be used to authenticate the client         */
/*                                                                           */
/*****************************************************************************/
typedef OSP_RC (OSPAPI * POSP_UM_CALLBACK)(OSP_CORRELATOR corr,
                                           char *timestamp,
                                           char *role,
                                           char *transaction_id,
                                           OSP_CALL_ID *call_id,
                                           OSP_ADDRESS *source_info,
                                           OSP_ADDRESS *source_alt_list,
                                           OSP_ADDRESS *dest_info,
                                           OSP_ADDRESS *dest_alt_list,
                                           OSP_USAGE_DETAIL *use_detail_list,
                                           OSP_CLIENT_ID *client_id);

/*****************************************************************************/
/* Name:    osp_um_response                                                  */
/*                                                                           */
/* Purpose: Called by the UM application in response to a Usage Metering     */
/*          callback from the OSP Server.                                    */
/*                                                                           */
/* Returns: OSP_SUCCESS - completed successfully                             */
/*          OSP_ERR_BAD_PARAM                                                */
/*                      - the _corr_ parameter is not a valid correlator     */
/*                        parameter (_corr_ should be exactly as passed to   */
/*                        the application in the UM callback), or the        */
/*                        _status_ parameter is NULL                         */
/*                                                                           */
/* Params:    IN     corr         - The correlator that was supplied by the  */
/*                                  stack on the corresponding USE callback  */
/*                                  invocation.                              */
/*            IN     status       - Pointer to an OSP_STATUS structure that  */
/*                                  specifies the status code and            */
/*                                  description that should be used to form  */
/*                                  the UsageConfirmation response.          */
/*            IN     audit_signal - This parameter controls the use of       */
/*                                  Cisco's AuditSignal extension to OSP.    */
/*                                  Set it to 0 if OpenOSP should not        */
/*                                  include an AuditSignal element in the    */
/*                                  response.  Otherwise, the following      */
/*                                  values are valid:                        */
/*                                  - OSP_UM_AUDIT_START: include an         */
/*                                    AuditSignal element in the response    */
/*                                    that tells the client to sign future   */
/*                                    UsageIndication messages using S/MIME  */
/*                                  - OSP_UM_AUDIT_STOP: include an          */
/*                                    AuditSignal element in the response    */
/*                                    that tells the client not to sign      */
/*                                    future UsageIndication messages using  */
/*                                    S/MIME.                                */
/*            IN     sig_required - Indicates whether the OSP server stack   */
/*                                  should S/MIME sign the response.         */
/*                                                                           */
/*****************************************************************************/
OSP_RC OSPAPI osp_um_response(OSP_CORRELATOR corr,
                              OSP_STATUS * status,
                              unsigned int audit_signal,
                              unsigned int sig_required);

/*****************************************************************************/
/* Name:    osp_um_register                                                  */
/*                                                                           */
/* Purpose: Called by the UM application to register its callback            */
/*          with the OSP Server.                                             */
/*                                                                           */
/*          Only one UM callback is supported per server.                    */
/*                                                                           */
/*          The application may pass a NULL pointer to deregister its        */
/*          callback.                                                        */
/*                                                                           */
/* Returns: OSP_SUCCESS         - completed successfully                     */
/*          OSP_ERR_NOT_INIT    - OSP Server has not yet been initialized    */
/*                                                                           */
/* Params:  um_cb - pointer to usage metering callback                       */
/*                                                                           */
/*****************************************************************************/
OSP_RC OSPAPI osp_um_register(POSP_UM_CALLBACK um_cb);

/*****************************************************************************/
/*                                                                           */
/* Authorization and Routing API                                             */
/*                                                                           */
/* The Authorization and Routing (AR) API authorizes calls and generates     */
/* routes and authorization tokens.  The API also allows the OSP server to   */
/* update pricing information based on OSP pricing indications.              */
/*                                                                           */
/* The API is asynchronous, that is responses may be sent by the             */
/* application after the original callback has returned, and may be sent on  */
/* a different thread.                                                       */
/*****************************************************************************/

/*****************************************************************************/
/* Name:    osp_ar_auth_rq_callback                                          */
/*                                                                           */
/* Purpose: Called by the OSP Server to pass a client Authorization Request  */
/*          to the AR application.                                           */
/*                                                                           */
/* Returns: OSP_SUCCESS - completed successfully                             */
/*          other       - OSP result code                                    */
/*                                                                           */
/* Params:  corr            - correlator to be used on subsequent response   */
/*          timestamp       - message timestamp                              */
/*          call_id_list    - list of call identifiers to use for returned   */
/*                            destinations                                   */
/*          source_info     - pointer to source info structure               */
/*          source_alt_list - source alternate list                          */
/*          dest_info       - pointer to destination info structure          */
/*          dest_alt_list   - destination alternate list                     */
/*          service         - service information                            */
/*          max_dests       - maximum number of destinations to be returned  */
/*          token_list      - list of authorization tokens which apply to    */
/*                            all destinations                               */
/*          authentication_info                                              */
/*                          - list of subscriber authentication information  */
/*          client_id       - may be used to authenticate the client         */
/*                                                                           */
/*****************************************************************************/
typedef OSP_RC (OSPAPI * POSP_AR_AUTH_RQ_CALLBACK)
                                   (OSP_CORRELATOR corr,
                                    char *timestamp,
                                    OSP_CALL_ID *call_id_list,
                                    OSP_ADDRESS *source_info,
                                    OSP_ADDRESS *source_alt_list,
                                    OSP_ADDRESS *dest_info,
                                    OSP_ADDRESS *dest_alt_list,
                                    OSP_SERVICE_INFO *service,
                                    char *max_dests,
                                    OSP_TOKEN *token_list,
                                    OSP_AUTH_INFO *authentication_info,
                                    OSP_CLIENT_ID *client_id);

/*****************************************************************************/
/* Name:    osp_ar_auth_response                                             */
/*                                                                           */
/* Purpose: Called by the AR application in response to an Authorization     */
/*          Request callback from the OSP Server.                            */
/*                                                                           */
/* Returns: OSP_SUCCESS - completed successfully                             */
/*                                                                           */
/* Params:  corr            - correlator supplied on corresponding callback  */
/*          status          - OSP status information for the operttion       */
/*          transaction_id  - transaction identifier                         */
/*          token_list      - list of authorization tokens which apply to    */
/*                            all destinations                               */
/*          dest_list       - destination alternate list                     */
/*          sig_required    - set to 1 if the server should S/MIME sign the  */
/*                            response, 0 otherwise                          */
/*                                                                           */
/*****************************************************************************/
OSP_RC OSPAPI osp_ar_auth_response(OSP_CORRELATOR corr,
                                   OSP_STATUS * status,
                                   char *transaction_id,
                                   OSP_TOKEN *token_list,
                                   OSP_DESTINATION *dest_list,
                                   unsigned int sig_required);

/*****************************************************************************/
/* Name:    osp_ar_auth_ind_callback                                         */
/*                                                                           */
/* Purpose: Called by the OSP Server to pass a client Authorization          */
/*          Indication to the AR application.                                */
/*                                                                           */
/* Returns: OSP_SUCCESS - completed successfully                             */
/*          other       - OSP result code                                    */
/*                                                                           */
/* Params:  corr            - correlator to be used on subsequent response   */
/*          timestamp       - message timestamp                              */
/*          role            - client role in call                            */
/*          call_id         - structure containing H.323 CallId              */
/*          source_info     - pointer to source info structure               */
/*          source_alt_list - source alternate list                          */
/*          dest_info       - pointer to destination info structure          */
/*          dest_alt_list   - list of destination alternates                 */
/*          service         - service information                            */
/*          auth_token_list - list of authorization tokens                   */
/*          client_id       - may be used to authenticate the client         */
/*                                                                           */
/*****************************************************************************/
typedef OSP_RC (OSPAPI * POSP_AR_AUTH_IND_CALLBACK)
                                      (OSP_CORRELATOR corr,
                                       char *timestamp,
                                       char *role,
                                       OSP_CALL_ID *call_id,
                                       OSP_ADDRESS *source_info,
                                       OSP_ADDRESS *source_alt_list,
                                       OSP_ADDRESS *dest_info,
                                       OSP_ADDRESS *dest_alt_list,
                                       OSP_SERVICE_INFO *service,
                                       OSP_TOKEN *auth_token_list,
                                       OSP_CLIENT_ID *client_id);

/*****************************************************************************/
/* Name:    osp_ar_auth_confirm                                              */
/*                                                                           */
/* Purpose: Called by the AR application in response to an Authorization     */
/*          Indication callback from the OSP Server.                         */
/*                                                                           */
/* Returns: OSP_SUCCESS - completed successfully                             */
/*                                                                           */
/* Params:  corr            - correlator supplied on corresponding callback  */
/*          status          - OSP status information for the operttion       */
/*          valid_after     - time authorization begins; NULL pointer means  */
/*                            immediately                                    */
/*          valid_until     - duration of authorization; NULL pointer means  */
/*                            indefinitely                                   */
/*          sig_required    - set to 1 if the server should S/MIME sign the  */
/*                            response, 0 otherwise                          */
/*                                                                           */
/*****************************************************************************/
OSP_RC OSPAPI osp_ar_auth_confirm(OSP_CORRELATOR corr,
                                  OSP_STATUS * status,
                                  char *valid_after,
                                  char *valid_until,
                                  unsigned int sig_required);

/*****************************************************************************/
/* Name:    osp_ar_reauth_rq_callback                                        */
/*                                                                           */
/* Purpose: Called by the OSP Server to pass a client Reauthorization        */
/*          Request to the AR application.                                   */
/*                                                                           */
/* Returns: OSP_SUCCESS - completed successfully                             */
/*          other       - OSP result code                                    */
/*                                                                           */
/* Params:  corr            - correlator to be used on subsequent response   */
/*          timestamp       - message timestamp                              */
/*          role            - client role in call                            */
/*          call_id         - structure containing H.323 CallId              */
/*          source_info     - pointer to source info structure               */
/*          source_alt_list - source alternate list                          */
/*          dest_info       - pointer to destination info structure          */
/*          dest_alt_list   - list of destination alternates                 */
/*          transaction_id  - transaction identifier                         */
/*          use_detail_list - list of usage detail structures                */
/*          token_list      - list of authorization tokens                   */
/*          client_id       - may be used to authenticate the client         */
/*                                                                           */
/*****************************************************************************/
typedef OSP_RC (OSPAPI * POSP_AR_REAUTH_RQ_CALLBACK)
                                     (OSP_CORRELATOR corr,
                                      char *timestamp,
                                      char *role,
                                      OSP_CALL_ID *call_id,
                                      OSP_ADDRESS *source_info,
                                      OSP_ADDRESS *source_alt_list,
                                      OSP_ADDRESS *dest_info,
                                      OSP_ADDRESS *dest_alt_list,
                                      char *transaction_id,
                                      OSP_USAGE_DETAIL *use_detail_list,
                                      OSP_TOKEN *token_list,
                                      OSP_CLIENT_ID *client_id);

/*****************************************************************************/
/* Name:    osp_ar_reauth_response                                           */
/*                                                                           */
/* Purpose: Called by the AR applicasion in response to an Reauthorization   */
/*          Request callback from the OSP Server.                            */
/*                                                                           */
/* Returns: OSP_SUCCESS - completed successfully                             */
/*                                                                           */
/* Params:  corr            - correlator supplied on corresponding callback  */
/*          status          - OSP status information for the operttion       */
/*          transaction_id  - transaction identifier                         */
/*          token_list      - list of authorization tokens which apply to    */
/*                            all destinations                               */
/*          dest_list       - list of destinations to which service is       */
/*                            authorized                                     */
/*          sig_required    - set to 1 if the server should S/MIME sign the  */
/*                            response, 0 otherwise                          */
/*                                                                           */
/*****************************************************************************/
OSP_RC OSPAPI osp_ar_reauth_response(OSP_CORRELATOR corr,
                                     OSP_STATUS * status,
                                     char *transaction_id,
                                     OSP_TOKEN *token_list,
                                     OSP_DESTINATION *dest_list,
                                     unsigned int sig_required);

/*****************************************************************************/
/* Name:    osp_ar_pricing_ind_callback                                      */
/*                                                                           */
/* Purpose: Called by the OSP Server to pass a Pricing Indication to the     */
/*          AR application.                                                  */
/*                                                                           */
/* Returns: OSP_SUCCESS - completed successfully                             */
/*          other       - OSP result code                                    */
/*                                                                           */
/* Params:  corr          - correlator to be used on subsequent response     */
/*          timestamp     - message timestamp                                */
/*          source_info   - pointer to source info structure                 */
/*          dest_info     - pointer to destination info structure            */
/*          currency      - currency used for pricing                        */
/*          amount        - currency per increment of time, packets etc      */
/*          increment     - number of units                                  */
/*          unit          - unit type - time, packets, bytes etc             */
/*          service       - service information                              */
/*          valid_after   - time new pricing begins; NULL pointer means      */
/*                          immediately                                      */
/*          valid_until   - duration of pricing validity; NULL pointer       */
/*                          means indefinitely                               */
/*          client_id     - may be used to authenticate the client           */
/*                                                                           */
/*****************************************************************************/
typedef OSP_RC (OSPAPI * POSP_AR_PRICING_IND_CALLBACK)
                            (OSP_CORRELATOR corr,
                             char *timestamp,
                             OSP_ADDRESS *source_info,
                             OSP_ADDRESS *dest_info,
                             char *currency,
                             char *amount,
                             char *increment,
                             char *unit,
                             OSP_SERVICE_INFO *service,
                             char *valid_after,
                             char *valid_until,
                             OSP_CLIENT_ID *client_id);

/*****************************************************************************/
/* Name:    osp_ar_pricing_confirm                                           */
/*                                                                           */
/* Purpose: Called by the AR application in response to a Pricing            */
/*          Indication callback from the OSP Server.                         */
/*                                                                           */
/* Returns: OSP_SUCCESS - completed successfully                             */
/*                                                                           */
/* Params:  corr            - correlator supplied on corresponding callback  */
/*          status          - OSP status information for the operttion       */
/*          sig_required    - set to 1 if the server should S/MIME sign the  */
/*                            response, 0 otherwise                          */
/*                                                                           */
/*****************************************************************************/
OSP_RC OSPAPI osp_ar_pricing_confirm(OSP_CORRELATOR corr,
                                     OSP_STATUS * status,
                                     unsigned int sig_required);

/*****************************************************************************/
/* Name:    osp_ar_caps_ind_callback                                         */
/*                                                                           */
/* Purpose: Called by the OSP Server to pass a Capabilities Indication to    */
/*          the AR application.                                              */
/*                                                                           */
/* Returns: OSP_SUCCESS - completed successfully                             */
/*          other       - OSP result code                                    */
/*                                                                           */
/* Params:  corr          - correlator to be used on subsequent response     */
/*          dev_info_list - list of information about the client device      */
/*          osp_version   - highest version of the OSP protocol the client   */
/*                          supports                                         */
/*          caps_list     - list of OSP components supported by the client   */
/*          resource_list - list of the OSP resources at the client          */
/*          client_id     - may be used to authenticate the client           */
/*                                                                           */
/*****************************************************************************/
typedef OSP_RC (OSPAPI * POSP_AR_CAPS_IND_CALLBACK)
                            (OSP_CORRELATOR corr,
                             OSP_DEVICE_INFO *dev_info_list,
                             char *osp_version,
                             OSP_CAPS *caps_list,
                             OSP_RESOURCE *resource_list,
                             OSP_CLIENT_ID *client_id);

/*****************************************************************************/
/* Name:    osp_ar_caps_confirm                                              */
/*                                                                           */
/* Purpose: Called by the AR application in response to a Capabilities       */
/*          Indication callback from the OSP Server.                         */
/*                                                                           */
/* Returns: OSP_SUCCESS - completed successfully                             */
/*          other       - OSP result code                                    */
/*                                                                           */
/* Params:  corr            - correlator supplied on corresponding callback  */
/*          status          - OSP status information for the operttion       */
/*          osp_version     - negotiated version of the OSP protocol to use  */
/*          service_list    - list of OSP services supported by the server   */
/*          cert_list       - list of public key certificates                */
/*          device_id       - server device identifier                       */
/*          sig_required    - set to 1 if the server should S/MIME sign the  */
/*                            response, 0 otherwise                          */
/*                                                                           */
/*****************************************************************************/
OSP_RC OSPAPI osp_ar_caps_confirm(OSP_CORRELATOR corr,
                                  OSP_STATUS * status,
                                  char *osp_version,
                                  OSP_SERVICE *service_list,
                                  OSP_CERTIFICATE *cert_list,
                                  char *device_id,
                                  unsigned int sig_required);

/*****************************************************************************/
/* Name:    osp_ar_register                                                  */
/*                                                                           */
/* Purpose: Called by the AR application to register its callbacks           */
/*          with the OSP Server.                                             */
/*                                                                           */
/*          Only one AR callback of each type is supported per server.       */
/*                                                                           */
/*          The AR application may pass a NULL pointer for any callback      */
/*          which it does not support.                                       */
/*                                                                           */
/* Returns: OSP_SUCCESS         - completed successfully                     */
/*                                                                           */
/* Params:  ar_cb - pointer to authorization request callback                */
/*          ai_cb - pointer to authorization indication callback             */
/*          ra_cb - pointer to reauthorization request callback              */
/*          pi_cb - pointer to pricing indication callback                   */
/*          cp_cb - pointer to capabilities indication callback              */
/*                                                                           */
/*****************************************************************************/
OSP_RC OSPAPI osp_ar_register(POSP_AR_AUTH_RQ_CALLBACK ar_cb,
                              POSP_AR_AUTH_IND_CALLBACK ai_cb,
                              POSP_AR_REAUTH_RQ_CALLBACK ra_cb,
                              POSP_AR_PRICING_IND_CALLBACK pi_cb,
                              POSP_AR_CAPS_IND_CALLBACK cp_cb);

/*****************************************************************************/
/*                                                                           */
/* Subscriber Authentication API                                             */
/*                                                                           */
/* The Subscriber Authentication (SA) API authenticates subscribers and      */
/* generates authentication tokens.                                          */
/*                                                                           */
/* The API is asynchronous, that is responses may be sent by the             */
/* application after the original callback has returned, and may be sent on  */
/* a different thread.                                                       */
/*****************************************************************************/

/*****************************************************************************/
/* Name:    osp_sa_callback                                                  */
/*                                                                           */
/* Purpose: Called by the OSP Server to pass a Subscriber Authentication     */
/*          request to the SA application.                                   */
/*                                                                           */
/* Returns: OSP_SUCCESS - completed successfully                             */
/*          other       - OSP result code                                    */
/*                                                                           */
/* Params:  corr            - correlator to be used on subsequent response   */
/*          timestamp       - message timestamp                              */
/*          source_info     - pointer to source info structure               */
/*          source_alt_list - source alternate list                          */
/*          dest_info       - pointer to destination info structure          */
/*          service         - service information                            */
/*          authentication_info                                              */
/*                          - subscriber authentication information          */
/*          client_id       - may be used to authenticate the client         */
/*                                                                           */
/*****************************************************************************/
typedef OSP_RC (OSPAPI * POSP_SA_CALLBACK) (OSP_CORRELATOR corr,
                                            char *timestamp,
                                            OSP_ADDRESS *source_info,
                                            OSP_ADDRESS *source_alt_list,
                                            OSP_ADDRESS *dest_info,
                                            OSP_SERVICE_INFO *service,
                                            OSP_AUTH_INFO *subs_authen_info,
                                            OSP_CLIENT_ID *client_id);

/*****************************************************************************/
/* Name:    osp_sa_response                                                  */
/*                                                                           */
/* Purpose: Called by the SA application in response to a Subscriber         */
/*          Authentication callback from the OSP Server.                     */
/*                                                                           */
/* Returns: OSP_SUCCESS - completed successfully                             */
/*                                                                           */
/* Params:  corr            - correlator supplied on corresponding callback  */
/*          status          - OSP status information for the operttion       */
/*          subs_token_list - list of subscriber authentication tokens       */
/*          credit_amount   - amount of credit available to subscriber       */
/*          credit_time     - time available based on credit amount          */
/*          sig_required    - set to 1 if the server should S/MIME sign the  */
/*                            response, 0 otherwise                          */
/*                                                                           */
/*                                                                           */
/*****************************************************************************/
OSP_RC OSPAPI osp_sa_response(OSP_CORRELATOR corr,
                              OSP_STATUS * status,
                              OSP_TOKEN *subs_token_list,
                              OSP_CREDIT_AMOUNT *credit_amount,
                              OSP_CREDIT_TIME *credit_time,
                              unsigned int sig_required);

/*****************************************************************************/
/* Name:    osp_sa_register                                                  */
/*                                                                           */
/* Purpose: Called by the SA application to register its SA callback         */
/*          with the OSP Server.                                             */
/*                                                                           */
/*          Only one SA callback is supported per server.                    */
/*                                                                           */
/*          The application may pass a NULL pointer to deregister its        */
/*          callback.                                                        */
/*                                                                           */
/* Returns: OSP_SUCCESS         - completed successfully                     */
/*                                                                           */
/* Params:  sa_cb - pointer to subscriber authentication callback            */
/*                                                                           */
/*****************************************************************************/
OSP_RC OSPAPI osp_sa_register(POSP_SA_CALLBACK sa_cb);

/*****************************************************************************/
/* Name:    osp_client_verify_callback                                       */
/*                                                                           */
/* Purpose: Called by the OSP Server to pass information about clients that  */
/*          connect to the OpenOSP stack, including certificates received    */
/*          during SSL/TLS session negotiation.  If no hook function is      */
/*          provided, all non-secure connections are accepted and all        */
/*          secure connections are accepted subject to a consistent and      */
/*          within-date certificate chain.                                   */
/*                                                                           */
/* Returns: OSP_SUCCESS    - client credentials were acceptable              */
/*          OSP_ERR_DENIED - credentials were not acceptable; connection     */
/*                           should be denied                                */
/*                                                                           */
/* Params:  ip_address  - the IP address of the originating client           */
/*          ver_cookie_ptr                                                   */
/*                      - a pointer to an application-supplied 'cookie'      */
/*                        that the server will supply on subsequent          */
/*                        application callbacks in the client id structure   */
/*          cert        - a pointer to an ASN.1 DER-encoded degenerate       */
/*                        PKCS#7 SignedData structure containing the X.509   */
/*                        certificate to be checked                          */
/*          cert_len    - the length of cert                                 */
/*          chain_len   - the total number of certificates in the chain,     */
/*                        including the clients certificate and the CA       */
/*                        certificate                                        */
/*          chain_pos   - the position in the chain of the supplied          */
/*                        certificate; 0 indicates the clients certificate   */
/*                                                                           */
/*****************************************************************************/
typedef OSP_RC (OSPAPI *POSP_CLIENT_VERIFY_CALLBACK)
                                              (unsigned long ip_address,
                                               void **ver_cookie_ptr,
                                               char *cert,
                                               unsigned int cert_len,
                                               unsigned int chain_len,
                                               unsigned int chain_pos);

/*****************************************************************************/
/* Name:    osp_client_disconnect_callback                                   */
/*                                                                           */
/* Purpose: Called by the OSP Server to notify the application that the      */
/*          connection to the client has closed, enabling the application    */
/*          to free any resources associated with that connection.           */
/*                                                                           */
/* Returns: None.                                                            */
/*                                                                           */
/* Params:  client_id - A structure containing the client IP address and     */
/*                      the cookie from any earlier client verification      */
/*                                                                           */
/*****************************************************************************/
typedef void (OSPAPI *POSP_CLIENT_DISCONNECT_CALLBACK)
                                                   (OSP_CLIENT_ID *client_id);

/*****************************************************************************/
/* Name:    osp_client_verify_register                                       */
/*                                                                           */
/* Purpose: Called by the application to register its client verification    */
/*          callback with the OSP Server.                                    */
/*                                                                           */
/*          Only one CV callback is supported per server.                    */
/*                                                                           */
/*          The application may pass a NULL pointer to deregister its        */
/*          callback.                                                        */
/*                                                                           */
/* Returns: OSP_SUCCESS         - completed successfully                     */
/*                                                                           */
/* Params:  cv_cb - pointer to the verification callback                     */
/*          cd_cb - pointer to the disconnection callback                    */
/*                                                                           */
/*****************************************************************************/
OSP_RC OSPAPI osp_client_verify_register
                                      (POSP_CLIENT_VERIFY_CALLBACK cv_cb,
                                       POSP_CLIENT_DISCONNECT_CALLBACK cd_cb);

/*****************************************************************************/
/* Name:    osp_non_repudiation_callback                                     */
/*                                                                           */
/* Purpose: An application-provided function that receives details of all    */
/*          S/MIME-signed OSP requests processed by the stack.  This         */
/*          function must not block.  It must not modify the contents of,    */
/*          or store a pointer to, the buffer passed                         */
/*                                                                           */
/* Returns: None.                                                            */
/*                                                                           */
/* Params:  buffer    - pointer to the complete HTTP request received        */
/*                      from the client                                      */
/*          buf_len   - length of buffer                                     */
/*          client_id - A structure containing the client IP address and     */
/*                      the cookie from any earlier client verification      */
/*                                                                           */
/*****************************************************************************/
typedef void (OSPAPI * POSP_NON_REPUDIATION_CALLBACK)
                                                   (char *buffer,
                                                    unsigned int buf_len,
                                                    OSP_CLIENT_ID *client_id);

/*****************************************************************************/
/* Name:    osp_non_repudiation_register                                     */
/*                                                                           */
/* Purpose: Called by the application to register its non-repudiation        */
/*          callback with the OSP Server.                                    */
/*                                                                           */
/*          Only one NR callback is supported per server.                    */
/*                                                                           */
/*          The application may pass a NULL pointer to deregister its        */
/*          callback.                                                        */
/*                                                                           */
/* Returns: OSP_SUCCESS         - completed successfully                     */
/*                                                                           */
/* Params:  nr_cb - pointer to the NR callback                               */
/*                                                                           */
/*****************************************************************************/
OSP_RC OSPAPI osp_non_repudiation_register
                                        (POSP_NON_REPUDIATION_CALLBACK nr_cb);

#ifdef __cplusplus
}
#endif

#endif

