#ifndef SCEPSTRC_INCLUDED
#define SCEPSTRC_INCLUDED

/**INC+***********************************************************************/
/* Header:    scepstrc.h                                                     */
/*                                                                           */
/* Purpose:   SCEP structure definitions                                     */
/*                                                                           */
/* Copyright (c) 2000, 2001 Data Connection Limited                          */
/* Copyright (c) 2000, 2001 Cisco Systems, Inc. All Rights Reserved.         */
/*                                                                           */
/* All software incorporated in the "OpenOSP Server" (except for those       */
/* portions incorporating third party software specifically identified as    */
/* being licensed under separate license) are licensed under the terms of    */
/* the Cisco Public Source License ver. 1.0 (2000) found in                  */
/* $OSPROOT/license.txt.  Additional copies of the license are available     */
/* from opensource@cisco.com.                                                */
/*                                                                           */
/* Also, before using this software, please see $OSPROOT/x509_ipr.txt for    */
/* information regarding intellectual property rights.                       */
/*                                                                           */
/* $Revision::   1.4                $ $Modtime::   Jan 11 2001 15:56:40   $  */
/*                                                                           */
/**INC-***********************************************************************/

/**STRUCT+********************************************************************/
/* Structure: SCEP_DATA                                                      */
/*                                                                           */
/* Description: SCEP internal data structure                                 */
/*****************************************************************************/

typedef struct scep_data
{
  /***************************************************************************/
  /* The SCEP CA's distiguished name.                                        */
  /***************************************************************************/
  char *ca_name;

  /***************************************************************************/
  /* Pointer to the CA certificate (in ASN.1 DER-encoded X.509 format)       */
  /***************************************************************************/
  char *ca_cert;

  /***************************************************************************/
  /* Length of CA certificate.                                               */
  /***************************************************************************/
  unsigned int ca_cert_len;

  /***************************************************************************/
  /* OpenSSL OID handle for messageType.                                     */
  /***************************************************************************/
  int nid_messageType;

  /***************************************************************************/
  /* OpenSSL OID handle for pkiStatus.                                       */
  /***************************************************************************/
  int nid_pkiStatus;

  /***************************************************************************/
  /* OpenSSL OID handle for failInfo.                                        */
  /***************************************************************************/
  int nid_failInfo;

  /***************************************************************************/
  /* OpenSSL OID handle for senderNonce.                                     */
  /***************************************************************************/
  int nid_senderNonce;

  /***************************************************************************/
  /* OpenSSL OID handle for recipientNonce.                                  */
  /***************************************************************************/
  int nid_recipientNonce;

  /***************************************************************************/
  /* OpenSSL OID handle for transId.                                         */
  /***************************************************************************/
  int nid_transId;

  /***************************************************************************/
  /* OpenSSL OID handle for extensionReq.                                    */
  /***************************************************************************/
  int nid_extensionReq;

} SCEP_DATA;

/**STRUCT-********************************************************************/

/**STRUCT+********************************************************************/
/* Structure: SCEP_STATE                                                     */
/*                                                                           */
/* Description: State for a SCEP transaction.                                */
/*                                                                           */
/*****************************************************************************/

typedef struct scep_state
{
  /***************************************************************************/
  /* The associated transaction control structure.                           */
  /***************************************************************************/
  TRANS_INFO *trans;

  /***************************************************************************/
  /* SCEP transaction ID of the received message.                            */
  /***************************************************************************/
  char *transaction_id;

  /***************************************************************************/
  /* Message type of the received message.                                   */
  /***************************************************************************/
  char *message_type;

  /***************************************************************************/
  /* senderNonce value and length from the received message.                 */
  /***************************************************************************/
  char *n_once;
  unsigned int n_once_len;

  /***************************************************************************/
  /* Sender's self-signed certificate and length from a PKCSReq, in ASN.1    */
  /* DER-encoded X.509 format.                                               */
  /***************************************************************************/
  char *ss_cert;
  unsigned int ss_cert_len;

} SCEP_STATE;

/**STRUCT-********************************************************************/

#endif

