#ifndef SMDEF_INCLUDED
#define SMDEF_INCLUDED

/**INC+***********************************************************************/
/* Header:    smdef.h                                                        */
/*                                                                           */
/* Purpose:   Definitions for SM                                             */
/*                                                                           */
/* Copyright (c) 2000, 2001 Data Connection Limited                          */
/* Copyright (c) 2000, 2001 Cisco Systems, Inc. All Rights Reserved.         */
/*                                                                           */
/* All software incorporated in the "OpenOSP Server" (except for those       */
/* portions incorporating third party software specifically identified as    */
/* being licensed under separate license) are licensed under the terms of    */
/* the Cisco Public Source License ver. 1.0 (2000) found in                  */
/* $OSPROOT/license.txt.  Additional copies of the license are available     */
/* from opensource@cisco.com.                                                */
/*                                                                           */
/* Also, before using this software, please see $OSPROOT/x509_ipr.txt for    */
/* information regarding intellectual property rights.                       */
/*                                                                           */
/* $Revision::   1.17               $ $Modtime::   Jan 11 2001 15:56:42   $  */
/*                                                                           */
/**INC-***********************************************************************/

/*****************************************************************************/
/* SM Configuration Constants                                                */
/*****************************************************************************/

/*****************************************************************************/
/* Time in seconds between SSL rengotiation                                  */
/*****************************************************************************/
#define SM_RENEGOTIATION_TIME     43200

/*****************************************************************************/
/* Maximum number of queued incoming connections on a listening socket       */
/*****************************************************************************/
#define SM_LISTEN_SOCKET_BACKLOG  5

/*****************************************************************************/
/* SSL session cache size                                                    */
/*****************************************************************************/
#define SM_SESSION_CACHE_SIZE     0

/*****************************************************************************/
/* Interval in microseconds to wait at a time on termination.                */
/*****************************************************************************/
#define SM_SLEEP_WAIT_INTERVAL    10000

/*****************************************************************************/
/* Initial size of receive buffers.                                          */
/*****************************************************************************/
#define SM_RECV_BUFFER_SIZE       4096

/*****************************************************************************/
/* Initial size of send buffers.                                             */
/*****************************************************************************/
#define SM_SEND_BUFFER_SIZE       4096

/*****************************************************************************/
/* The number of bytes that SM reserves at the start of a transaction's send */
/* buffer.  This must be large enough to accommodate any possible HTTP       */
/* headers that SM may prepend to an OSP or SCEP request that is about to be */
/* sent.  It is up to the S/MIME or SCEP layers to reserve additional space  */
/* to accomodate Content-Type values that they may pass down to us.          */
/*****************************************************************************/
#define SM_RESERVED_HEADER_SIZE    128

#if SM_RESERVED_HEADER_SIZE > SM_SEND_BUFFER_SIZE
#error Reserved header size must be less than or equal to send buffer size
#endif

/*****************************************************************************/
/* Constants required by config file reader                                  */
/*****************************************************************************/

/*****************************************************************************/
/* Default port for listening for incoming non-secure connections            */
/*****************************************************************************/
#define SM_DEFAULT_NONSECURE_PORT 80

/*****************************************************************************/
/* Default port for listening for incoming secure connections                */
/*****************************************************************************/
#define SM_DEFAULT_SECURE_PORT    443

/*****************************************************************************/
/* Default value for number of SM worker threads per processor.              */
/*****************************************************************************/
#define SM_DEFAULT_THREADS_PER_PROCESSOR    2

/*****************************************************************************/
/* Name of item in config file that defines secure listening port            */
/*****************************************************************************/
#define OSP_CONFIG_SM_SSL_PORT    "ssl_port"

/*****************************************************************************/
/* Name of item in config file that defines non-secure listening port        */
/*****************************************************************************/
#define OSP_CONFIG_SM_NONSSL_PORT "non_ssl_port"

/*****************************************************************************/
/* Name of item in config file that configures the number of SM worker       */
/* threads per processor.                                                    */
/*****************************************************************************/
#define OSP_CONFIG_SM_THREADS_PER_PROCESSOR    "threads_per_processor"

/*****************************************************************************/
/* Connection States                                                         */
/*****************************************************************************/
#define SM_CONN_STATE_OPEN         0
#define SM_CONN_STATE_DONT_RECEIVE 1
#define SM_CONN_STATE_CLOSED       2

/*****************************************************************************/
/* Constants returned by sm_worker_received_data()                           */
/*****************************************************************************/
#define SM_CALL_AGAIN        0
#define SM_DONT_CALL_AGAIN   1
#define SM_CONNECTION_CLOSED 2

/*****************************************************************************/
/* Invalid socket value                                                      */
/*****************************************************************************/
#define SM_INVALID_SOCKET -1

/*****************************************************************************/
/* Type of socket connection                                                 */
/*****************************************************************************/
#define SM_SECURE_SOCKET     0
#define SM_NON_SECURE_SOCKET 1

/*****************************************************************************/
/* Default select timeout period in seconds                                  */
/*****************************************************************************/
#define SM_SELECT_TIMEOUT    10

/*****************************************************************************/
/* Constants tracking the initialization state of the stack                  */
/*****************************************************************************/
#define OSP_INIT_NEVER  0
#define OSP_INIT_NONE   1
#define OSP_INIT_COM    2
#define OSP_INIT_RM     3
#define OSP_INIT_CONFIG 4
#define OSP_INIT_SMIME  5
#define OSP_INIT_STATS  6
#define OSP_INIT_SM     7
#define OSP_INIT_SCEP   8
#define OSP_INIT_FULL   9

/*****************************************************************************/
/* Constants tracking the initialization state of SM                         */
/*****************************************************************************/
#define SM_INIT_NONE              0
#define SM_INIT_SSL               1
#define SM_INIT_CCM               2
#define SM_INIT_CB_MUTEX          3
#define SM_INIT_WL_MUTEX          4
#define SM_INIT_PIPE              5
#define SM_INIT_NON_SECURE_SOCK   6
#define SM_INIT_SOCKETS           7
#define SM_INIT_FULL              8

/*****************************************************************************/
/* Macro to calculate nfds for a thread                                      */
/*****************************************************************************/
#define SM_RECALCULATE_NFDS(T)                                              \
{                                                                           \
  CONN_INFO *sm_conn;                                                       \
                                                                            \
  (T)->nfds = OSP_MAX((T)->notification_pipe_out, (T)->wakeup_pipe_out);    \
                                                                            \
  for (sm_conn = (CONN_INFO *)LIST_GET_FIRST(&(T)->conn_list);              \
       sm_conn != NULL;                                                     \
       sm_conn = (CONN_INFO *)LIST_GET_NEXT(&(T)->conn_list,                \
                                  &sm_conn->links))                         \
  {                                                                         \
    if ((T)->nfds < sm_conn->sock_handle)                                   \
    {                                                                       \
      (T)->nfds = sm_conn->sock_handle;                                     \
    }                                                                       \
  }                                                                         \
                                                                            \
  (T)->nfds++;                                                              \
}

/*****************************************************************************/
/* Macro to set transaction's http_version field, with checking of           */
/* unexpected happenings.                                                    */
/*****************************************************************************/
#define SM_SET_HTTP_VERSION(T, V)                                           \
do                                                                          \
{                                                                           \
  if ((T)->http_version == SM_HTTP_VERSION_UNDETECTED)                      \
  {                                                                         \
    TRC_DETAIL((TB, "Transaction HTTP version detected: http_version=%d",   \
                (V)));                                                      \
    (T)->connection->http_version = (V);                                    \
  }                                                                         \
  else if ((T)->http_version == (V))                                        \
  {                                                                         \
    TRC_DETAIL((TB, "Transaction HTTP version unchanged: http_version=%d",  \
                (V)));                                                      \
  }                                                                         \
  else                                                                      \
  {                                                                         \
    TRC_ERROR((TB, "Transaction HTTP version changing: current=%d new=%d",  \
               (T)->http_version, (V)));                                    \
  }                                                                         \
  (T)->http_version = (V);                                                  \
}                                                                           \
while (0)

#ifdef OSP_MEASURE_PERFORMANCE
/*****************************************************************************/
/* Performance measurement macro.                                            */
/*****************************************************************************/
#define SM_ACCUMULATE(gv, origin, now)                  \
do                                                      \
{                                                       \
  gv.count++;                                           \
  gv.accum.tv_sec += now.tv_sec - origin.tv_sec;        \
  gv.accum.tv_usec += now.tv_usec - origin.tv_usec;     \
  while (gv.accum.tv_usec < 0)                          \
  {                                                     \
    gv.accum.tv_sec--;                                  \
    gv.accum.tv_usec += 1000000;                        \
  }                                                     \
  while (gv.accum.tv_usec >= 1000000)                   \
  {                                                     \
    gv.accum.tv_sec++;                                  \
    gv.accum.tv_usec -= 1000000;                        \
  }                                                     \
}                                                       \
while (0)
#endif

#endif

