#ifndef SMGLBL_INCLUDED
#define SMGLBL_INCLUDED

/**INC+***********************************************************************/
/* Header:    smglbl.h                                                       */
/*                                                                           */
/* Purpose:   SM Global variabled                                            */
/*                                                                           */
/* Copyright (c) 2000, 2001 Data Connection Limited                          */
/* Copyright (c) 2000, 2001 Cisco Systems, Inc. All Rights Reserved.         */
/*                                                                           */
/* All software incorporated in the "OpenOSP Server" (except for those       */
/* portions incorporating third party software specifically identified as    */
/* being licensed under separate license) are licensed under the terms of    */
/* the Cisco Public Source License ver. 1.0 (2000) found in                  */
/* $OSPROOT/license.txt.  Additional copies of the license are available     */
/* from opensource@cisco.com.                                                */
/*                                                                           */
/* Also, before using this software, please see $OSPROOT/x509_ipr.txt for    */
/* information regarding intellectual property rights.                       */
/*                                                                           */
/* $Revision::   1.11               $ $Modtime::   Jan 11 2001 15:56:42   $  */
/*                                                                           */
/**INC-***********************************************************************/

/*****************************************************************************/
/* SM Listener data                                                          */
/*****************************************************************************/
#ifdef SM_DEFINE_VARS
LISTENER_INFO smg_listener_info;
#else
extern LISTENER_INFO smg_listener_info;
#endif

/*****************************************************************************/
/* Data read from config file                                                */
/*****************************************************************************/
#ifdef SM_DEFINE_VARS
CONFIG_DATA smg_config_data;
#else
extern CONFIG_DATA smg_config_data;
#endif

/*****************************************************************************/
/* Statistics - mutex protects statistics                                    */
/*****************************************************************************/
#ifdef SM_DEFINE_VARS
pthread_mutex_t smg_statistics_mutex;
OSP_STATS smg_statistics;
#else
extern pthread_mutex_t smg_statistics_mutex;
extern OSP_STATS smg_statistics;
#endif

/*****************************************************************************/
/* Flag to indicate that we've set the crypto memory functions.              */
/*****************************************************************************/
#ifdef SM_DEFINE_VARS
int smg_done_once_only_init = 0;
#else
extern int smg_done_once_only_init;
#endif

#ifdef OSP_MEASURE_TRANSACTION
/*****************************************************************************/
/* Transaction timings descriptions.                                         */
/*****************************************************************************/
#ifdef SM_DEFINE_VARS
const char *smg_timings_description[SM_NUM_TIMINGS] = {
  "Start read",
  "Finish read",
  "Start parse",
  "Finish parse",
  "Start build",
  "Finish build",
  "Start send",
  "Finish send"
};
#else
extern const char *smg_timings_description[];
#endif
#endif

/*****************************************************************************/
/* Accumulated transaction processing time, excluding I/O.                   */
/*****************************************************************************/
#ifdef SM_DEFINE_VARS
SM_PERF smg_processing = { "Transaction processing", 0, { 0, 0 } };
SM_PERF smg_until_smime_received = { "Until smime_rcvd_request", 0, { 0, 0 } };
SM_PERF smg_until_send_queue = { "Until added to send queue", 0, { 0, 0 } };
SM_PERF smg_until_start_send = { "Until starting to send", 0, { 0, 0 } };
SM_PERF smg_until_sent = { "Until response sent", 0, { 0, 0 } };
SM_PERF smg_select_cycle = { "Select cycle time", 0, { 0, 0 } };
SM_PERF smg_wake_up = { "Write wake up time", 0, { 0, 0 } };
#else
extern SM_PERF smg_processing;
extern SM_PERF smg_until_smime_received;
extern SM_PERF smg_until_send_queue;
extern SM_PERF smg_until_start_send;
extern SM_PERF smg_until_sent;
extern SM_PERF smg_select_cycle;
extern SM_PERF smg_wake_up;
#endif

/*****************************************************************************/
/* Thread-specific data key used to to identify SM threads                   */
/*****************************************************************************/
#ifdef SM_DEFINE_VARS
pthread_key_t sm_thread_key;
const char sm_key_name[] = "OpenOSP SM Thread";
#else
extern pthread_key_t sm_thread_key;
extern const char sm_key_name[];
#endif

#endif

