#ifndef SMICONST_INCLUDED
#define SMICONST_INCLUDED

/**INC+***********************************************************************/
/* Header:    smiconst.h                                                     */
/*                                                                           */
/* Purpose:   S/MIME constants                                               */
/*                                                                           */
/* Copyright (c) 2000, 2001 Data Connection Limited                          */
/* Copyright (c) 2000, 2001 Cisco Systems, Inc. All Rights Reserved.         */
/*                                                                           */
/* All software incorporated in the "OpenOSP Server" (except for those       */
/* portions incorporating third party software specifically identified as    */
/* being licensed under separate license) are licensed under the terms of    */
/* the Cisco Public Source License ver. 1.0 (2000) found in                  */
/* $OSPROOT/license.txt.  Additional copies of the license are available     */
/* from opensource@cisco.com.                                                */
/*                                                                           */
/* Also, before using this software, please see $OSPROOT/x509_ipr.txt for    */
/* information regarding intellectual property rights.                       */
/*                                                                           */
/* $Revision::   1.6                $ $Modtime::   Jan 11 2001 15:56:44   $  */
/*                                                                           */
/**INC-***********************************************************************/

/*****************************************************************************/
/* Configuration file value names.                                           */
/*****************************************************************************/
#ifdef SMIME_DEFINE_VARS
const char smime_config_identity[] = "smime_identity";
const char smime_config_default_digest[] = "smime_default_digest";
#else
extern const char smime_config_identity[];
extern const char smime_config_default_digest[];
#endif

/*****************************************************************************/
/* Header names to be parsed for multipart/signed parts.                     */
/*****************************************************************************/
#ifdef SMIME_DEFINE_VARS
const char smime_content_type[] = "Content-Type";
const char smime_content_length[] = "Content-Length";
#else
extern const char smime_content_type[];
extern const char smime_content_length[];
#endif

/*****************************************************************************/
/* Content-Type values.                                                      */
/*****************************************************************************/
#ifdef SMIME_DEFINE_VARS
const char smime_content_signed[] = "multipart/signed";
const char smime_content_plain[] = "text/plain";
const char smime_content_signature[] = "application/pkcs7-signature";
#else
extern const char smime_content_signed[];
extern const char smime_content_plain[];
extern const char smime_content_signature[];
#endif

/*****************************************************************************/
/* Content-Type parameters for multipart/signed.                             */
/*****************************************************************************/
#ifdef SMIME_DEFINE_VARS
const char smime_ct_param_protocol[] = "protocol";
const char smime_ct_param_micalg[] = "micalg";
const char smime_ct_param_boundary[] = "boundary";
#else
extern const char smime_ct_param_protocol[];
extern const char smime_ct_param_micalg[];
extern const char smime_ct_param_boundary[];
#endif

/*****************************************************************************/
/* Digest names in text form. LN_xxx are OpenSSL string constants.           */
/*****************************************************************************/
#ifdef SMIME_DEFINE_VARS
const char smime_digest_sha1[] = "sha1";
const char smime_digest_md5[] = "md5";
#else
extern const char smime_digest_sha1[];
extern const char smime_digest_md5[];
#endif

/*****************************************************************************/
/* Templates for use with sprintf() to construct MIME part headers and       */
/* boundaries.                                                               */
/*****************************************************************************/
#ifdef SMIME_DEFINE_VARS
const char smime_content_type_format[] = "multipart/signed; protocol=\"%s\"; "
                           "micalg=%s; boundary=\"" SMIME_BOUNDARY_FORMAT "\"";
const char smime_part_header_format[] = "\r\n--" SMIME_BOUNDARY_FORMAT "\r\n"
                                        "Content-Type: %s\r\n"
                                        "Content-Length: %u\r\n\r\n";
const char smime_end_boundary_format[] =
                                       "\r\n--" SMIME_BOUNDARY_FORMAT "--\r\n";
#else
extern const char smime_content_type_format[];
extern const char smime_part_header_format[];
extern const char smime_end_boundary_format[];
#endif

/*****************************************************************************/
/* Boundary delimiters.                                                      */
/*****************************************************************************/
#ifdef SMIME_DEFINE_VARS
const char smime_boundary_delimiter[] = "--";
#else
extern const char smime_boundary_delimiter[];
#endif

#endif

